#!/usr/bin/env python
# -*- coding: utf-8 -*-
import lief
import unittest
import logging
import os
import sys
import random

from unittest import TestCase
from utils import get_sample

from lief import Logger
Logger.set_level(lief.LOGGING_LEVEL.INFO)

class TestLoadConfig(TestCase):
    def setUp(self):
        self.logger = logging.getLogger(__name__)


    def test_winapp(self):
        winapp = lief.parse(get_sample('PE/PE64_x86-64_binary_WinApp.exe'))
        self.assertTrue(winapp.has_configuration)

        lconf = winapp.load_configuration

        self.assertEqual(lconf.version, lief.PE.WIN_VERSION.WIN10_0_15002)
        self.assertEqual(lconf.characteristics, 0xF8)
        self.assertEqual(lconf.timedatestamp, 0)
        self.assertEqual(lconf.major_version, 0)
        self.assertEqual(lconf.minor_version, 0)
        self.assertEqual(lconf.global_flags_clear, 0)
        self.assertEqual(lconf.global_flags_set, 0)
        self.assertEqual(lconf.critical_section_default_timeout, 0)
        self.assertEqual(lconf.decommit_free_block_threshold, 0)
        self.assertEqual(lconf.decommit_total_free_threshold, 0)
        self.assertEqual(lconf.lock_prefix_table, 0)
        self.assertEqual(lconf.maximum_allocation_size, 0)
        self.assertEqual(lconf.virtual_memory_threshold, 0)
        self.assertEqual(lconf.process_affinity_mask, 0)
        self.assertEqual(lconf.process_heap_flags, 0)
        self.assertEqual(lconf.csd_version, 0)
        self.assertEqual(lconf.reserved1, 0)
        self.assertEqual(lconf.editlist, 0)
        self.assertEqual(lconf.security_cookie, 0x4000d008)

        # V0
        self.assertEqual(lconf.se_handler_table, 0)
        self.assertEqual(lconf.se_handler_count, 0)

        # V1
        self.assertEqual(lconf.guard_cf_check_function_pointer, 0x140012000)
        self.assertEqual(lconf.guard_cf_dispatch_function_pointer, 0x140012010)
        self.assertEqual(lconf.guard_cf_function_table, 0x140011000)
        self.assertEqual(lconf.guard_cf_function_count, 15)

        expected_flags  = lief.PE.GUARD_CF_FLAGS.LONGJUMP_TABLE_PRESENT
        expected_flags |= lief.PE.GUARD_CF_FLAGS.FUNCTION_TABLE_PRESENT
        expected_flags |= lief.PE.GUARD_CF_FLAGS.INSTRUMENTED
        self.assertEqual(lconf.guard_flags, lief.PE.GUARD_CF_FLAGS(expected_flags))

        # V2
        code_integrity = lconf.code_integrity

        self.assertEqual(code_integrity.flags, 0)
        self.assertEqual(code_integrity.catalog, 0)
        self.assertEqual(code_integrity.catalog_offset, 0)
        self.assertEqual(code_integrity.reserved, 0)

        # V3
        self.assertEqual(lconf.guard_address_taken_iat_entry_table, 0)
        self.assertEqual(lconf.guard_address_taken_iat_entry_count, 0)
        self.assertEqual(lconf.guard_long_jump_target_table, 0)
        self.assertEqual(lconf.guard_long_jump_target_count, 0)

        # V4
        self.assertEqual(lconf.dynamic_value_reloc_table, 0)
        self.assertEqual(lconf.hybrid_metadata_pointer, 0)

        # V5
        self.assertEqual(lconf.guard_rf_failure_routine, 0x140001040)
        self.assertEqual(lconf.guard_rf_failure_routine_function_pointer, 0x140012020)
        self.assertEqual(lconf.dynamic_value_reloctable_offset, 0)
        self.assertEqual(lconf.dynamic_value_reloctable_section, 0)
        self.assertEqual(lconf.reserved2, 0)

        # V6
        self.assertEqual(lconf.guard_rf_verify_stackpointer_function_pointer, 0x140012030)
        self.assertEqual(lconf.hotpatch_table_offset, 0)



if __name__ == '__main__':

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    root_logger.addHandler(ch)

    unittest.main(verbosity=2)




#!/usr/bin/env python2
#-*- coding: utf-8 -*-

# This file is used to generate config file for LIEF test
# It use `pefile`(https://github.com/erocarrera/pefile) to generate yaml config file

import re
import yaml
import hashlib
import pefile
import sys
import os
import copy

def generate_node_config(node):
    node_y = dict()
    node_y["childs"] = []
    if type(node) == pefile.ResourceDirData:
        node_y["type"] = 0
        node_y["Characteristics"]      = node.struct.Characteristics
        node_y["TimeDateStamp"]        = node.struct.TimeDateStamp
        node_y["MajorVersion"]         = node.struct.MajorVersion
        node_y["MinorVersion"]         = node.struct.MinorVersion
        node_y["NumberOfNamedEntries"] = node.struct.NumberOfNamedEntries
        node_y["NumberOfIdEntries"]    = node.struct.NumberOfIdEntries
        for rsrc in node.entries:
            node_y["childs"].append(generate_node_config(rsrc))

    elif type(node) == pefile.ResourceDirEntryData:
        node_y["type"]          = 1
        node_y["Name"]          = str(node.name)
        node_y["id"]            = node.id
        node_y["OffsetToData"]  = node.struct.OffsetToData
        if hasattr(node, "directory"):
            node_y["childs"].append(generate_node_config(node.directory))
        else:
            node_y["childs"].append(generate_node_config(node.data))
    elif type(node) == pefile.ResourceDataEntryData:
        node_y["type"] = 2
        node_y["OffsetToData"] = node.struct.OffsetToData
        node_y["Size"]         = node.struct.Size
        node_y["CodePage"]     = node.struct.CodePage
        node_y["Reserved"]     = node.struct.Reserved
        node_y["Reserved"]     = node.struct.Reserved
        node_y["lang"]         = node.lang
        node_y["sublang"]      = node.sublang
    else:
        print "Unknown type"
        return

    return node_y

def generate_config(binary_path):
    pe     = pefile.PE(binary_path)

    binary = dict()

    dos_header = dict()

    dos_header["e_magic"]    = pe.DOS_HEADER.e_magic
    dos_header["e_cblp"]     = pe.DOS_HEADER.e_cblp
    dos_header["e_crlc"]     = pe.DOS_HEADER.e_crlc
    dos_header["e_cparhdr"]  = pe.DOS_HEADER.e_cparhdr
    dos_header["e_minalloc"] = pe.DOS_HEADER.e_minalloc
    dos_header["e_maxalloc"] = pe.DOS_HEADER.e_maxalloc
    dos_header["e_ss"]       = pe.DOS_HEADER.e_ss
    dos_header["e_sp"]       = pe.DOS_HEADER.e_sp
    dos_header["e_csum"]     = pe.DOS_HEADER.e_csum

    header = dict()

    header["Machine"]              = pe.FILE_HEADER.Machine
    header["NumberOfSections"]     = pe.FILE_HEADER.NumberOfSections
    header["TimeDateStamp"]        = pe.FILE_HEADER.TimeDateStamp
    header["PointerToSymbolTable"] = pe.FILE_HEADER.PointerToSymbolTable
    header["NumberOfSymbols"]      = pe.FILE_HEADER.NumberOfSymbols
    header["SizeOfOptionalHeader"] = pe.FILE_HEADER.SizeOfOptionalHeader
    header["Characteristics"]      = pe.FILE_HEADER.Characteristics

    optional_header = dict()

    optional_header["Magic"]                       = pe.OPTIONAL_HEADER.Magic
    optional_header["MajorLinkerVersion"]          = pe.OPTIONAL_HEADER.MajorLinkerVersion
    optional_header["MinorLinkerVersion"]          = pe.OPTIONAL_HEADER.MinorLinkerVersion
    optional_header["SizeOfCode"]                  = pe.OPTIONAL_HEADER.SizeOfCode
    optional_header["SizeOfInitializedData"]       = pe.OPTIONAL_HEADER.SizeOfInitializedData
    optional_header["SizeOfUninitializedData"]     = pe.OPTIONAL_HEADER.SizeOfUninitializedData
    optional_header["AddressOfEntryPoint"]         = pe.OPTIONAL_HEADER.AddressOfEntryPoint
    optional_header["BaseOfCode"]                  = pe.OPTIONAL_HEADER.BaseOfCode
    if pe.OPTIONAL_HEADER.Magic == pefile.OPTIONAL_HEADER_MAGIC_PE:
        optional_header["BaseOfData"]                  = pe.OPTIONAL_HEADER.BaseOfData
    optional_header["ImageBase"]                   = pe.OPTIONAL_HEADER.ImageBase
    optional_header["SectionAlignment"]            = pe.OPTIONAL_HEADER.SectionAlignment
    optional_header["FileAlignment"]               = pe.OPTIONAL_HEADER.FileAlignment
    optional_header["MajorOperatingSystemVersion"] = pe.OPTIONAL_HEADER.MajorOperatingSystemVersion
    optional_header["MinorOperatingSystemVersion"] = pe.OPTIONAL_HEADER.MinorOperatingSystemVersion
    optional_header["MajorImageVersion"]           = pe.OPTIONAL_HEADER.MajorImageVersion
    optional_header["MinorImageVersion"]           = pe.OPTIONAL_HEADER.MinorImageVersion
    optional_header["MajorSubsystemVersion"]       = pe.OPTIONAL_HEADER.MajorSubsystemVersion
    optional_header["MinorSubsystemVersion"]       = pe.OPTIONAL_HEADER.MinorSubsystemVersion
    optional_header["Reserved1"]                   = pe.OPTIONAL_HEADER.Reserved1
    optional_header["SizeOfImage"]                 = pe.OPTIONAL_HEADER.SizeOfImage
    optional_header["SizeOfHeaders"]               = pe.OPTIONAL_HEADER.SizeOfHeaders
    optional_header["CheckSum"]                    = pe.OPTIONAL_HEADER.CheckSum
    optional_header["Subsystem"]                   = pe.OPTIONAL_HEADER.Subsystem
    optional_header["DllCharacteristics"]          = pe.OPTIONAL_HEADER.DllCharacteristics
    optional_header["SizeOfStackReserve"]          = pe.OPTIONAL_HEADER.SizeOfStackReserve
    optional_header["SizeOfStackCommit"]           = pe.OPTIONAL_HEADER.SizeOfStackCommit
    optional_header["SizeOfHeapReserve"]           = pe.OPTIONAL_HEADER.SizeOfHeapReserve
    optional_header["SizeOfHeapCommit"]            = pe.OPTIONAL_HEADER.SizeOfHeapCommit
    optional_header["LoaderFlags"]                 = pe.OPTIONAL_HEADER.LoaderFlags
    optional_header["NumberOfRvaAndSizes"]         = pe.OPTIONAL_HEADER.NumberOfRvaAndSizes

    sections = []
    for section in pe.sections:
        sec = dict()
        sec["name"] = str(section.Name).replace("\0", "")
        sec["Misc_VirtualSize"]     = section.Misc_VirtualSize
        sec["VirtualAddress"]       = section.VirtualAddress
        sec["SizeOfRawData"]        = section.SizeOfRawData
        sec["PointerToRawData"]     = section.PointerToRawData
        sec["PointerToRelocations"] = section.PointerToRelocations
        sec["PointerToLinenumbers"] = section.PointerToLinenumbers
        sec["NumberOfRelocations"]  = section.NumberOfRelocations
        sec["NumberOfLinenumbers"]  = section.NumberOfLinenumbers
        sec["Characteristics"]      = section.Characteristics
        sections.append(sec)

    imports = []
    for entry in pe.DIRECTORY_ENTRY_IMPORT:
        import_ = dict()
        import_["name"]           = entry.dll
        import_["TimeDateStamp"]  = entry.struct.TimeDateStamp
        import_["ForwarderChain"] = entry.struct.ForwarderChain
        entries = []
        for imp in entry.imports:
            impo = dict()
            if not imp.import_by_ordinal:
                impo["name"] = imp.name
            else:
                impo["name"] = None

            entries.append(impo)
        import_["entries"] = entries
        imports.append(import_)

    tls = dict()
    if hasattr(pe, "DIRECTORY_ENTRY_TLS"):
        tls["StartAddressOfRawData"] = pe.DIRECTORY_ENTRY_TLS.struct.StartAddressOfRawData
        tls["EndAddressOfRawData"]   = pe.DIRECTORY_ENTRY_TLS.struct.EndAddressOfRawData
        tls["AddressOfIndex"]        = pe.DIRECTORY_ENTRY_TLS.struct.AddressOfIndex
        tls["AddressOfCallBacks"]    = pe.DIRECTORY_ENTRY_TLS.struct.AddressOfCallBacks
        tls["SizeOfZeroFill"]        = pe.DIRECTORY_ENTRY_TLS.struct.SizeOfZeroFill
        tls["Characteristics"]       = pe.DIRECTORY_ENTRY_TLS.struct.Characteristics

    resources = dict()
    if hasattr(pe, "DIRECTORY_ENTRY_RESOURCE"):
        node = pe.DIRECTORY_ENTRY_RESOURCE
        resources = generate_node_config(node)

    binary["dos_header"]      = dos_header
    binary["header"]          = header
    binary["optional_header"] = optional_header
    binary["sections"]        = sections
    binary["imports"]         = imports
    binary["tls"]             = tls
    binary["resources"]       = resources

    binary["filename"] = str(binary_path).split("/")[-1]
    binary["hash"]     = hashlib.md5(binary_path).hexdigest()

    output = open(binary_path + ".yaml", "w")
    yaml.dump(binary, stream=output)
    output.close()

if __name__ == "__main__":
    path_pe32_binaries = "./samples/PE/win32/"
    path_pe64_binaries = "./samples/PE/win64/"

    for binary in os.listdir(path_pe32_binaries):
        if binary.endswith(".exe") or binary.endswith(".dll"):
            print "[PE32] Dealing with:", binary
            generate_config(path_pe32_binaries + binary)

    for binary in os.listdir(path_pe64_binaries):
        if binary.endswith(".exe") or binary.endswith(".dll"):
            print "[PE64] Dealing with:", binary
            generate_config(path_pe64_binaries + binary)



#!/usr/bin/env python3.6

# Launch Docker CI when committing
# Inspired from https://medium.com/@zypherman/my-experiences-with-travis-ci-7f5ea26a87c0

import requests
import json
import time
import sys
import logging
import pprint
import os

LOG_LEVEL = logging.INFO

logging.getLogger().addHandler(logging.StreamHandler())
logging.getLogger().setLevel(LOG_LEVEL)
logger = logging.getLogger(__name__)


AUTH_TOKEN = os.getenv("TRAVIS_TOKEN", sys.argv[1] if len(sys.argv) > 1 else None)

if not AUTH_TOKEN:
    logger.fatal("No token found!")
    sys.exit(1)

LIEF_ID   = 12579556
DOCKER_ID = 15139981

ENDPOINT = 'https://api.travis-ci.org'

LIEF   = f"{ENDPOINT}/repo/{LIEF_ID}"
DOCKER = f"{ENDPOINT}/repo/{DOCKER_ID}"

DOCKER_ENV_VARS_ENDPOINT = f"{DOCKER}/env_vars"

headers = {
    'Content-Type':       'application/json',
    'Accept':             'application/json',
    'User-Agent':         'API Explorer',
    'Travis-API-Version': '3',
    'Authorization':      f'token {AUTH_TOKEN}'
}

def last_docker_build():
    response = requests.get(f"{DOCKER}/branch/master", headers=headers)

    if not response.ok:
        logging.error("Error while getting last_docker_build")
        sys.exit(1)

    href = response.json()["last_build"]["@href"]
    build_info = requests.get(f"{ENDPOINT}{href}", headers=headers)

    if not build_info.ok:
        logging.error("Error while getting build_info")
        sys.exit(1)

    return build_info.json()



def trigger_build():
    response = requests.get(f"{DOCKER}/branch/master", headers=headers)
    build_info = last_docker_build()

    pprint.pprint(build_info)

    # Get last build info
    last_build_id     = build_info["id"]
    last_build_status = build_info["state"]

    # Create env variables
    env_vars = [
        {
            "env_var.name": "TRIGGER_COMMIT",
            "env_var.value": os.getenv("TRAVIS_COMMIT"),
            "env_var.public": False,
        },
        {
            "env_var.name": "TRIGGER_REPO",
            "env_var.value": os.getenv("TRAVIS_REPO_SLUG"),
            "env_var.public": False,
        },
        {
            "env_var.name": "TRIGGER_ACTION",
            "env_var.value": "build-doc",
            "env_var.public": False,
        },
        {
            "env_var.name": "DEPENDENT_BUILD",
            "env_var.value": "true",
            "env_var.public": False,
        },
    ]

    env_vars_results = requests.get(DOCKER_ENV_VARS_ENDPOINT, headers=headers)
    if not env_vars_results.ok:
        logger.error("Error while getting Docker env variables")
        sys.exit(1)
    env_vars_remote = env_vars_results.json()["env_vars"]

    # Set env variables
    for env_var in env_vars:
        remote_var = next(filter(lambda v : v["name"] == env_var["env_var.name"], env_vars_remote), None)
        if remote_var:
            r = requests.patch(f"{DOCKER}/env_var/{remote_var['id']}", data=json.dumps(env_var), headers=headers)
            if not r.ok:
                loggger.error(f"Error while setting variable: '{remote_var['name']}'")
                sys.exit(1)
        else:
            result = requests.post(DOCKER_ENV_VARS_ENDPOINT, data=json.dumps(env_var), headers=headers)
            if not result.ok:
                loggger.error(f"Error while setting variable: '{env_var['env_var.name']}'")
                sys.exit(1)

    # (Re)Start build
    result = requests.post(f"{ENDPOINT}/build/{last_build_id}/restart", headers=headers)
    if not result.ok:
        logger.error("Error while restarting the build")
        sys.exit(1)

    return 0



def main():
    is_pull_request = not (os.getenv("TRAVIS_PULL_REQUEST", "false") == "false")

    if is_pull_request:
        logger.info("Skipping pull request build...")
        return 0

    return trigger_build()




if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
import unittest
import logging
import os
import sys
import stat
import re
import subprocess
import tempfile
import shutil
from subprocess import Popen

import lief
from lief.ELF import Segment
from lief import Logger
#Logger.set_level(lief.LOGGING_LEVEL.DEBUG)

from unittest import TestCase
from utils import get_sample

CURRENT_DIRECTORY = os.path.dirname(os.path.abspath(__file__))

class TestAddSegment(TestCase):
    def setUp(self):
        self.logger = logging.getLogger(__name__)
        self.tmp_dir = tempfile.mkdtemp(suffix='_lief_test_add_segment')
        self.logger.debug("temp dir: {}".format(self.tmp_dir))


    @unittest.skipUnless(sys.platform.startswith("linux"), "requires Linux")
    def test_simple(self):
        sample_path = get_sample('ELF/ELF64_x86-64_binary_ls.bin')
        stub        = lief.parse(os.path.join(CURRENT_DIRECTORY, "hello_lief.bin"))
        output      = os.path.join(self.tmp_dir, "ls.segment")

        target                  = lief.parse(sample_path)
        for i in range(4):
            segment                 = stub.segments[0]
            original_va             = segment.virtual_address
            segment.virtual_address = 0
            segment                 = target.add(segment)
            new_ep                  = (stub.header.entrypoint - original_va) + segment.virtual_address

            target.header.entrypoint = new_ep
        target.write(output)

        st = os.stat(output)
        os.chmod(output, st.st_mode | stat.S_IEXEC)

        p = Popen(output, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        stdout, _ = p.communicate()
        self.logger.debug(stdout.decode("utf8"))
        self.assertIsNotNone(re.search(r'LIEF is Working', stdout.decode("utf8")))


    @unittest.skipUnless(sys.platform.startswith("linux"), "requires Linux")
    def test_gcc(self):
        sample_path = get_sample('ELF/ELF64_x86-64_binary_gcc.bin')
        stub        = lief.parse(os.path.join(CURRENT_DIRECTORY, "hello_lief.bin"))
        output      = os.path.join(self.tmp_dir, "gcc.segment")

        target                  = lief.parse(sample_path)
        segment                 = stub.segments[0]
        original_va             = segment.virtual_address
        segment.virtual_address = 0
        segment                 = target.add(segment)
        new_ep                  = (stub.header.entrypoint - original_va) + segment.virtual_address

        target.header.entrypoint = new_ep
        target.write(output)

        st = os.stat(output)
        os.chmod(output, st.st_mode | stat.S_IEXEC)

        p = Popen(output, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        stdout, _ = p.communicate()
        self.logger.debug(stdout.decode("utf8"))
        self.assertIsNotNone(re.search(r'LIEF is Working', stdout.decode("utf8")))


    @unittest.skipUnless(sys.platform.startswith("linux"), "requires Linux")
    def test_static(self):
        sample_path = get_sample('ELF/ELF64_x86-64_binary_static-binary.bin')
        stub        = lief.parse(os.path.join(CURRENT_DIRECTORY, "hello_lief.bin"))
        output      = os.path.join(self.tmp_dir, "static.segment")

        target                  = lief.parse(sample_path)
        segment                 = stub.segments[0]
        original_va             = segment.virtual_address
        segment.virtual_address = 0
        segment                 = target.add(segment)
        new_ep                  = (stub.header.entrypoint - original_va) + segment.virtual_address

        target.header.entrypoint = new_ep
        target.write(output)

        st = os.stat(output)
        os.chmod(output, st.st_mode | stat.S_IEXEC)

        p = Popen(output, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        stdout, _ = p.communicate()
        self.logger.debug(stdout.decode("utf8"))
        self.assertIsNotNone(re.search(r'LIEF is Working', stdout.decode("utf8")))

    @unittest.skipUnless(sys.platform.startswith("linux"), "requires Linux")
    def test_misc(self):
        list_binaries = [
        '/usr/bin/ls',
        '/usr/bin/ssh',
        '/usr/bin/nm',
        '/usr/bin/openssl',
        '/usr/bin/bc',
        '/usr/bin/bzip2',
        '/usr/bin/cp',
        '/usr/bin/find',
        '/usr/bin/file',
        ]
        for binary in list_binaries:
            self.logger.debug("Test with '{}'".format(binary))
            self.run_add_segment(binary)


    def run_add_segment(self, target):
        if not os.path.isfile(target):
            return

        name   = os.path.basename(target)
        stub   = lief.parse(os.path.join(CURRENT_DIRECTORY, "hello_lief.bin"))
        target = lief.parse(target)
        output = os.path.join(self.tmp_dir, "{}.segment".format(name))
        for i in range(6):
            segment                 = stub.segments[0]
            original_va             = segment.virtual_address
            segment.virtual_address = 0
            segment                 = target.add(segment)
            new_ep                  = (stub.header.entrypoint - original_va) + segment.virtual_address

            target.header.entrypoint = new_ep
        target.write(output)

        st = os.stat(output)
        os.chmod(output, st.st_mode | stat.S_IEXEC)

        p = Popen(output, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        stdout, _ = p.communicate()
        self.logger.debug(stdout.decode("utf8"))
        self.assertIsNotNone(re.search(r'LIEF is Working', stdout.decode("utf8")))

    @unittest.skipUnless(False, "requires Linux")
    def test_libc(self):
        stub = lief.parse(os.path.join(CURRENT_DIRECTORY, "hello_lief.bin"))
        tmp_dir = tempfile.mkdtemp(suffix='_lief_test_add_segment_libc')
        self.logger.debug("temp dir: {}".format(tmp_dir))

        libc_name = "libc.so.6"
        for e in lief.parse("/bin/ls").libraries:
            if e.startswith("libc."):
                libc_name = e
                break;

        self.logger.debug("libc used: {}".format(libc_name))


        libc = lief.parse('/usr/lib/{}'.format(libc_name))
        out = os.path.join(tmp_dir, libc_name)

        for i in range(10):
            segment = stub.segments[0]
            original_va = segment.virtual_address
            segment.virtual_address = 0

            segment = libc.add(segment)

            new_ep = (stub.header.entrypoint - original_va) + segment.virtual_address

            if libc.has(lief.ELF.DYNAMIC_TAGS.INIT_ARRAY):
                init_array = libc.get(lief.ELF.DYNAMIC_TAGS.INIT_ARRAY)
                callbacks = init_array.array
                callbacks[0] = new_ep
                init_array.array = callbacks

            if libc.has(lief.ELF.DYNAMIC_TAGS.INIT):
                init = libc.get(lief.ELF.DYNAMIC_TAGS.INIT)
                init.value = new_ep

        libc.write(out)

        st = os.stat(out)
        os.chmod(out, st.st_mode | stat.S_IEXEC)

        p = Popen(["/usr/bin/ls"], stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env={"LD_LIBRARY_PATH": tmp_dir})
        stdout, _ = p.communicate()
        self.logger.debug(stdout.decode("utf8"))

        self.assertIsNotNone(re.search(r'LIEF is Working', stdout.decode("utf8")))

        if os.path.isdir(tmp_dir):
            shutil.rmtree(tmp_dir)




    def tearDown(self):
        # Delete it
        if os.path.isdir(self.tmp_dir):
            shutil.rmtree(self.tmp_dir)

if __name__ == '__main__':

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    root_logger.addHandler(ch)

    unittest.main(verbosity=2)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import lief
import unittest
import logging
import tempfile
import shutil
import os
import sys
import stat
import subprocess
import time
import ctypes
import zipfile
import json

from subprocess import Popen

from unittest import TestCase
from utils import get_sample

class TestPe(TestCase):
    def setUp(self):
        self.logger = logging.getLogger(__name__)
        self.maxDiff = None


    def test_code_view_pdb(self):
        path = get_sample('PE/PE64_x86-64_binary_ConsoleApplication1.exe')
        sample = lief.parse(path)

        self.assertTrue(sample.has_debug)

        debug = sample.debug

        self.assertTrue(debug.has_code_view)

        code_view = debug.code_view

        self.assertEqual(code_view.cv_signature, lief.PE.CODE_VIEW_SIGNATURES.PDB_70)
        self.assertEqual(code_view.signature, [245, 217, 227, 182, 71, 113, 1, 79, 162, 3, 170, 71, 124, 74, 186, 84])
        self.assertEqual(code_view.age, 1)
        self.assertEqual(code_view.filename, r"c:\users\romain\documents\visual studio 2015\Projects\HelloWorld\x64\Release\ConsoleApplication1.pdb")

        json_view = json.loads(lief.to_json(debug))
        self.assertDictEqual(json_view, {
            'addressof_rawdata': 8996,
            'characteristics': 0,
            'code_view': {
                'age': 1,
                'cv_signature': 'PDB_70',
                'filename': 'c:\\users\\romain\\documents\\visual studio 2015\\Projects\\HelloWorld\\x64\\Release\\ConsoleApplication1.pdb',
                'signature': [245, 217, 227, 182, 71, 113, 1, 79, 162, 3, 170, 71, 124, 74, 186, 84]
            },
            'major_version': 0,
            'minor_version': 0,
            'pointerto_rawdata': 5412,
            'sizeof_data': 125,
            'timestamp': 1459952944,
            'type': 'CODEVIEW'
        })




if __name__ == '__main__':

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    root_logger.addHandler(ch)

    unittest.main(verbosity=2)




#!/usr/bin/env python
import unittest
import lief
import logging
import pprint
import json
import os

from lief import Logger
Logger.set_level(lief.LOGGING_LEVEL.DEBUG)
#Logger.set_level(lief.LOGGING_LEVEL.DEBUG)

from unittest import TestCase
from utils import get_sample

CURRENT_DIR = os.path.abspath(os.path.dirname(__file__))

class TestART(TestCase):

    def setUp(self):
        self.logger = logging.getLogger(__name__)

    def test_art17(self):
        boot = lief.ART.parse(get_sample("ART/ART_017_AArch64_boot.art"))
        print(boot.header)
        return

    def test_art29(self):
        boot = lief.ART.parse(get_sample("ART/ART_029_ARM_boot.art"))

        print(boot.header)
        return

    def test_art30(self):
        boot = lief.ART.parse(get_sample("ART/ART_030_AArch64_boot.art"))

        print(boot.header)
        return

    def test_art44(self):
        boot = lief.ART.parse(get_sample("ART/ART_044_ARM_boot.art"))

        print(boot.header)
        return

    def test_art46(self):
        boot = lief.ART.parse(get_sample("ART/ART_046_AArch64_boot.art"))

        print(boot.header)
        return




if __name__ == '__main__':

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    root_logger.addHandler(ch)

    unittest.main(verbosity=2)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import lief
import unittest
import logging
import os
import sys
import stat
import re
import subprocess
import tempfile
import shutil
import time
import ctypes

from subprocess import Popen

from unittest import TestCase
from utils import get_sample

class TestHooking(TestCase):
    def setUp(self):
        self.logger = logging.getLogger(__name__)
        self.tmp_dir = tempfile.mkdtemp(suffix='_lief_test_hooking')
        self.logger.debug("temp dir: {}".format(self.tmp_dir))

        if sys.platform.startswith("win"):
            SEM_NOGPFAULTERRORBOX = 0x0002 # From MSDN
            ctypes.windll.kernel32.SetErrorMode(SEM_NOGPFAULTERRORBOX);


    def test_hook_64(self):
        sample_path = get_sample('PE/PE64_x86-64_binary_HelloWorld.exe')
        output      = os.path.join(self.tmp_dir, "pe64_hooking.exe")

        hello = lief.parse(sample_path)
        code = [
                0x48, 0x83, 0xc4, 0x48,                                     # add rsp, 0x48         ; Stack unwind
                0x48, 0x31, 0xc9,                                           # xor rcx, rcx          ; hWnd
                0x48, 0x89, 0xd2,                                           # mov rdx, rdx          ; Message
                0x49, 0xb8, 0x00, 0x90, 0x00, 0x40, 0x01, 0x00, 0x00, 0x00, # mov r8,  0x0140009000 ; Title
                0x4d, 0x31, 0xc9,                                           # xor r9, r9            ; MB_OK
                0x48, 0xb8, 0xe4, 0xa3, 0x00, 0x40, 0x01, 0x00, 0x00, 0x00, # mov rax, 0x014000A3E4 ; MessageBoxA address
                0xff, 0x10,                                                 # call [rax]            ; MessageBoxA(hWnd, Message, Title, MB_OK)
                0x48, 0x31, 0xc9,                                           # xor rcx, rcx          ; exit value
                0x48, 0xb8, 0xd4, 0xa3, 0x00, 0x40, 0x01, 0x00, 0x00, 0x00, # mov rax, 0x014000A3d4 ; ExitProcess address
                0xff, 0x10,                                                 # call [rax]            ; ExitProcess(0)
                0xc3,                                                       # ret                   ; Never reached
                ]

        title = "LIEF is awesome\0"

        data =  list(map(ord, title))

        section_text                 = lief.PE.Section(".htext")
        section_text.content         = code
        section_text.virtual_address = 0x8000
        section_text.characteristics = lief.PE.SECTION_CHARACTERISTICS.CNT_CODE | lief.PE.SECTION_CHARACTERISTICS.MEM_READ | lief.PE.SECTION_CHARACTERISTICS.MEM_EXECUTE

        section_data                 = lief.PE.Section(".hdata")
        section_data.content         = data
        section_data.virtual_address = 0x9000
        section_data.characteristics = lief.PE.SECTION_CHARACTERISTICS.CNT_INITIALIZED_DATA | lief.PE.SECTION_CHARACTERISTICS.MEM_READ

        hello.optional_header.dll_characteristics &= ~lief.PE.DLL_CHARACTERISTICS.DYNAMIC_BASE
        hello.optional_header.dll_characteristics &= ~lief.PE.DLL_CHARACTERISTICS.NX_COMPAT

        section_text = hello.add_section(section_text)
        section_data = hello.add_section(section_data)

        kernel32 = hello.get_import("KERNEL32.dll")
        kernel32.add_entry("ExitProcess")

        user32 = hello.add_library("user32.dll")
        user32.add_entry("MessageBoxA")

        ExitProcess_addr = hello.predict_function_rva("KERNEL32.dll", "ExitProcess")
        MessageBoxA_addr = hello.predict_function_rva("user32.dll", "MessageBoxA")
        self.logger.debug("Address of 'MessageBoxA': 0x{:06x} ".format(MessageBoxA_addr))
        self.logger.debug("Address of 'ExitProcess': 0x{:06x} ".format(ExitProcess_addr))

        hello.hook_function("__acrt_iob_func", section_text.virtual_address + hello.optional_header.imagebase)

        builder = lief.PE.Builder(hello)
        builder.build_imports(True).patch_imports(True)
        builder.build()

        builder.write(output)

        st = os.stat(output)
        os.chmod(output, st.st_mode | stat.S_IEXEC)

        if sys.platform.startswith("win"):
            subprocess_flags = 0x8000000 # win32con.CREATE_NO_WINDOW?
            p = Popen(["START", output, "foo"], shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, creationflags=subprocess_flags)
            time.sleep(3)
            q = Popen(["taskkill", "/im", "pe64_hooking.exe"], shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

            stdout, _ = p.communicate()
            self.logger.debug(stdout.decode("utf8"))

            stdout, _ = q.communicate()
            self.logger.debug(stdout.decode("utf8"))

            self.assertEqual(q.returncode, 0)

    def test_hook_32(self):
        sample_path = get_sample('PE/PE32_x86_binary_HelloWorld.exe')
        output      = os.path.join(self.tmp_dir, "pe32_hooking.exe")

        hello = lief.parse(sample_path)
        code = [
                0x6a, 0x00,                   # push 0               ; hWnd
                0x68, 0x00, 0x80, 0x40, 0x00, # push 0x408000        ; Title
                0x8b, 0x4d, 0x0c,             # mov ecx, [ebp + 0xC] ; Message
                0x51,                         # push ecx             ;
                0x6a, 0x00,                   # push 0               ; MB_OK
                0xb8, 0x3c, 0x92, 0x40, 0x00, # mov eax, 0x40923C    ; MessageBoxA address
                0xff, 0x10,                   # call [eax]           ; MessageBoxA(hWnd, Message, Title, MB_OK)
                0x6a, 0x00,                   # push 0               ; exit value
                0xb8, 0x34, 0x92, 0x40, 0x00, # mov eax, 0x409234    ; ExitProcess address
                0xff, 0x10,                   # call [eax]           ; ExitProcess(0)
                0xc3,                         # ret                  ; Never reached
                ]

        title = "LIEF is awesome\0"

        data =  list(map(ord, title))

        section_text                 = lief.PE.Section(".htext")
        section_text.content         = code
        section_text.virtual_address = 0x7000
        section_text.characteristics = lief.PE.SECTION_CHARACTERISTICS.CNT_CODE | lief.PE.SECTION_CHARACTERISTICS.MEM_READ | lief.PE.SECTION_CHARACTERISTICS.MEM_EXECUTE

        section_data                 = lief.PE.Section(".hdata")
        section_data.content         = data
        section_data.virtual_address = 0x8000
        section_data.characteristics = lief.PE.SECTION_CHARACTERISTICS.CNT_INITIALIZED_DATA | lief.PE.SECTION_CHARACTERISTICS.MEM_READ

        hello.optional_header.dll_characteristics &= ~lief.PE.DLL_CHARACTERISTICS.DYNAMIC_BASE
        hello.optional_header.dll_characteristics &= ~lief.PE.DLL_CHARACTERISTICS.NX_COMPAT

        section_text = hello.add_section(section_text)
        section_data = hello.add_section(section_data)

        kernel32 = hello.get_import("KERNEL32.dll")
        kernel32.add_entry("ExitProcess")

        user32 = hello.add_library("user32.dll")
        user32.add_entry("MessageBoxA")

        ExitProcess_addr = hello.predict_function_rva("KERNEL32.dll", "ExitProcess")
        MessageBoxA_addr = hello.predict_function_rva("user32.dll", "MessageBoxA")
        self.logger.debug("Address of 'MessageBoxA': 0x{:06x} ".format(MessageBoxA_addr))
        self.logger.debug("Address of 'ExitProcess': 0x{:06x} ".format(ExitProcess_addr))

        hello.hook_function("__acrt_iob_func", section_text.virtual_address + hello.optional_header.imagebase)

        builder = lief.PE.Builder(hello)
        builder.build_imports(True).patch_imports(True)
        builder.build()

        builder.write(output)

        st = os.stat(output)
        os.chmod(output, st.st_mode | stat.S_IEXEC)

        if sys.platform.startswith("win"):
            subprocess_flags = 0x8000000 # win32con.CREATE_NO_WINDOW?
            p = Popen(["START", output, "foo"], shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, creationflags=subprocess_flags)
            time.sleep(3)
            q = Popen(["taskkill", "/im", "pe32_hooking.exe"], shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

            stdout, _ = p.communicate()
            self.logger.debug(stdout.decode("utf8"))

            stdout, _ = q.communicate()
            self.logger.debug(stdout.decode("utf8"))

            self.assertEqual(q.returncode, 0)





    def tearDown(self):
        # Delete it
        try:
            if os.path.isdir(self.tmp_dir):
                shutil.rmtree(self.tmp_dir)
        except Exception as e:
            self.logger.error(e)


if __name__ == '__main__':

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    root_logger.addHandler(ch)

    unittest.main(verbosity=2)



#!/usr/bin/env python
import unittest
import logging
import os
import sys
import stat
import re
import subprocess
import tempfile
import shutil
from subprocess import Popen

import lief
from lief.ELF import Section

from unittest import TestCase
from utils import get_sample

CURRENT_DIRECTORY = os.path.dirname(os.path.abspath(__file__))
STUB = lief.parse(os.path.join(CURRENT_DIRECTORY, "hello_lief.bin"))

LIBADD_C = """\
#include <stdlib.h>
#include <stdio.h>

int add(int a, int b);

int add(int a, int b) {
  printf("%d + %d = %d\\n", a, b, a+b);
  return a + b;
}
"""

BINADD_C = """\
#include <stdio.h>
#include <stdlib.h>

int add(int a, int b);

int main(int argc, char **argv) {
  if (argc != 3) {
    printf("Usage: %s <a> <b>\\n", argv[0]);
    exit(-1);
  }

  int res = add(atoi(argv[1]), atoi(argv[2]));
  printf("From myLIb, a + b = %d\\n", res);
  return 0;
}
"""


class TestAddContent(TestCase):
    def setUp(self):
        self.logger = logging.getLogger(__name__)
        self.tmp_dir = tempfile.mkdtemp(suffix='_lief_test_content')
        self.logger.debug("temp dir: {}".format(self.tmp_dir))

        self.binadd_path = os.path.join(self.tmp_dir, "binadd.c")
        self.libadd_path = os.path.join(self.tmp_dir, "libadd.c")

        self.libadd_so  = os.path.join(self.tmp_dir, "libadd.so")
        self.binadd_bin = os.path.join(self.tmp_dir, "binadd.bin")

        # self.compiler = shutil.which(os.environ.get('CC', 'cc')) # only for Python > 3
        self.compiler = '/usr/bin/cc'
        if self.compiler is None:
            self.logger.error("Unable to find a compiler")
            sys.exit(0)

        self.logger.debug("Compiler: {}".format(self.compiler))

        with open(self.binadd_path, 'w') as f:
            f.write(BINADD_C)

        with open(self.libadd_path, 'w') as f:
            f.write(LIBADD_C)

    def compile_libadd(self, input, extra_flags=[]):
        if os.path.isfile(self.libadd_so):
            os.remove(self.libadd_so)
        CC_FLAGS = ['-fPIC', '-shared'] + extra_flags
        cmd = [self.compiler, '-o', self.libadd_so] + CC_FLAGS + [input]
        self.logger.debug("Compile 'libadd' with: {}".format(" ".join(cmd)))
        p = Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        stdout, _ = p.communicate()
        self.logger.debug(stdout)


    def compile_binadd(self, input, extra_flags=[]):
        if os.path.isfile(self.binadd_bin):
            os.remove(self.binadd_bin)

        CC_FLAGS = ['-L', self.tmp_dir] + extra_flags
        cmd = [self.compiler, '-o', self.binadd_bin] + CC_FLAGS + [input, '-ladd']
        self.logger.debug("Compile 'binadd' with: {}".format(" ".join(cmd)))
        p = Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        stdout, _ = p.communicate()
        self.logger.debug(stdout)


    @unittest.skipUnless(sys.platform.startswith("linux"), "requires Linux")
    def test_simple(self):
        self.compile_libadd(self.libadd_path)
        self.compile_binadd(self.binadd_path)

        libadd = lief.parse(self.libadd_so)
        for i in range(10):
            segment = libadd.add(STUB.segments[0])

            new_ep = (STUB.header.entrypoint - STUB.segments[0].virtual_address) + segment.virtual_address

            if libadd.has(lief.ELF.DYNAMIC_TAGS.INIT_ARRAY):
                init_array = libadd.get(lief.ELF.DYNAMIC_TAGS.INIT_ARRAY)
                callbacks = init_array.array
                callbacks[0] = new_ep
                init_array.array = callbacks

            if libadd.has(lief.ELF.DYNAMIC_TAGS.INIT):
                init = libadd.get(lief.ELF.DYNAMIC_TAGS.INIT)
                init.value = new_ep

        libadd.write(self.libadd_so)

        st = os.stat(self.libadd_so)
        os.chmod(self.libadd_so, st.st_mode | stat.S_IEXEC)

        p = Popen([self.binadd_bin, '1', '2'], stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env={"LD_LIBRARY_PATH": self.tmp_dir})
        stdout, _ = p.communicate()
        self.logger.debug(stdout.decode("utf8"))

        self.assertIsNotNone(re.search(r'LIEF is Working', stdout.decode("utf8")))

    def tearDown(self):
        # Delete it
        if os.path.isdir(self.tmp_dir):
            shutil.rmtree(self.tmp_dir)

if __name__ == '__main__':

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    root_logger.addHandler(ch)

    unittest.main(verbosity=2)




#!/usr/bin/env python2
#-*- coding: utf-8 -*-

# This file is used to generate config file for LIEF test
# Basically it parse it parse the output of readelf

import re
import yaml
import subprocess
import sys
import hashlib

p = subprocess.Popen(["readelf", "-a", sys.argv[1]], stdout=subprocess.PIPE)
(output, err) = p.communicate()

data = output
binary = dict()
header = dict()

#ELF64
elf64_regex = {
        "entrypoint"    : "Adresse du point d'entrée:\s+0x([0-9a-f]+)",
        "sectionoffset" : "Début des en-têtes de section\s*:\s+([0-9]+)",
        "offsetToPhdr"  : "Début des en-têtes de programme :\s+([0-9]+)",
        "nbShdr"        : "Nombre d'en-têtes de section\s*:\s+([0-9]+)",
        "nbPhdr"        : "Nombre d'en-tête du programme\s*:\s+([0-9]+)",

        "sections"      : ur'\[\s*(\d+)\]\s?(\S+|\s*)\s+\S+\s+([0-9A-Fa-f]+)\s+([0-9A-Fa-f]+)\n\s+([0-9A-Fa-f]+)\s+([0-9A-Fa-f]+)',
        "segments"      : ur'\s+(\w+)\s+0x([0-9A-Fa-f]+)\s+0x([0-9A-Fa-f]+)\s+0x([0-9A-Fa-f]+)\n\s+0x([0-9A-Fa-f]+)\s+0x([0-9A-Fa-f]+)',

        "dynamicSymGlob": ur'^T(?:.*)\.dynsym.*\n((?:\s{2,}(?:.*)\n)+)',
        "dynamicSymLoc" : ur'([0-9]+):\s+[0-9a-fA-F]+\s+[0-9]+\s+\S+\s+\S+\s+\S+\s+\S+\s?([^@\n]*)',

        "staticSymGlob" : ur'^T(?:.*)\.symtab.*\n((?:.*\n)+)$\s',
        "staticSymLoc"  : ur'([0-9]+):\s+[0-9a-fA-F]+\s+[0-9]+\s+\S+\s+\S+\s+\S+\s+\S+ (\S*)[@\n]+',

        "dynRelocaGlob" : ur'^S(?:.*)\.rel[a]*\.dyn.*\n((?:.{2,}(?:.*)\n)+)',
        "dynRelocaLoc"  : ur'^([0-9a-fA-F]+)\s+([0-9a-fA-F]+)\s+\S+\s+([0-9a-fA-F]+)\s*(?:\n|(?:\s+(\S+)))',

        "pltRelocaGlob" : ur'^S(?:.*)\.rel[a]*\.plt.*\n((?:.{2,}(?:.*)\n)+)',
        "pltRelocaLoc"  : ur'^([0-9a-fA-F]+)\s+([0-9a-fA-F]+)\s+\S+\s+([0-9a-fA-F]+)\s*(?:\n|(?:\s+(\S+)))'
    }


# ELF32
elf64_regex = {
        "entrypoint"    : "Adresse du point d'entrée:\s+0x([0-9a-f]+)",
        "sectionoffset" : "Début des en-têtes de section\s*:\s+([0-9]+)",
        "offsetToPhdr"  : "Début des en-têtes de programme :\s+([0-9]+)",
        "nbShdr"        : "Nombre d'en-têtes de section\s*:\s+([0-9]+)",
        "nbPhdr"        : "Nombre d'en-tête du programme\s*:\s+([0-9]+)",

        "sections"      : ur'\s+\[\s*([0-9]+)\]\s?(\s|\S+)\s+\S+\s+([0-9A-Fa-f]+)\s+([0-9A-Fa-f]+)\s+([0-9A-Fa-f]+)\s+([0-9A-Fa-f]+)',
        "segments"      : ur'\s+(\w+)\s+0x([0-9A-Fa-f]+)\s+0x([0-9A-Fa-f]+)\s0x([0-9A-Fa-f]+)\s0x([0-9A-Fa-f]+)\s0x([0-9A-Fa-f]+)',

        "dynamicSymGlob": ur'^T(?:.*)\.dynsym.*\n((?:\s{2,}(?:.*)\n)+)',
        "dynamicSymLoc" : ur'([0-9]+):\s+[0-9a-fA-F]+\s+[0-9]+\s+\S+\s+\S+\s+\S+\s+\S+\s?([^@\n]*)',

        "staticSymGlob" : ur'^T(?:.*)\.symtab.*\n((?:.*\n)+)$\s',
        "staticSymLoc"  : ur'([0-9]+):\s+[0-9a-fA-F]+\s+[0-9]+\s+\S+\s+\S+\s+\S+\s+\S+ (\S*)[@\n]+',

        "dynRelocaGlob" : ur'^S(?:.*)\.rel[a]*\.dyn.*\n((?:.{2,}(?:.*)\n)+)',
        "dynRelocaLoc"  : ur'^([0-9a-fA-F]+)\s+([0-9a-fA-F]+)\s+\S+\s+([0-9a-fA-F]+)\s*(?:\n|(?:\s+([^@\n]+)))',

        "pltRelocaGlob" : ur'^S(?:.*)\.rel[a]*\.plt.*\n((?:.{2,}(?:.*)\n)+)',
        "pltRelocaLoc"  : ur'^([0-9a-fA-F]+)\s+([0-9a-fA-F]+)\s+\S+\s+([0-9a-fA-F]+)\s*(?:\n|(?:\s+([^@\n]+)))'
    }




#
# File info
#
binary["filename"] = str(sys.argv[1]).split("/")[-1]
binary["hash"]     = hashlib.md5(sys.argv[1]).hexdigest()
#path: "@CMAKE_CURRENT_SOURCE_DIR@/samples/ELF/x86-64/binaries/ls"

#
# header
#
entrypoint    = re.search(elf64_regex["entrypoint"], data).groups()[0]
sectionoffset = re.search(elf64_regex["sectionoffset"], data).groups()[0]
offsetToPhdr  = re.search(elf64_regex["offsetToPhdr"], data).groups()[0]
nbShdr        = re.search(elf64_regex["nbShdr"], data).groups()[0]
nbPhdr        = re.search(elf64_regex["nbPhdr"], data).groups()[0]
header['entryPoint']   = int(entrypoint, 16)
header['offsetToShdr'] = int(sectionoffset)
header['offsetToPhdr'] = int(offsetToPhdr)
header['nbShdr']       = int(nbShdr)
header['nbPhdr']       = int(nbPhdr)

#
# Sections
#
section_regexp   = re.compile(elf64_regex["sections"], re.MULTILINE)
sections_yaml = []
sections = re.findall(section_regexp, data)
for section in sections:
    section_yaml = {
            'nb'     : int(section[0]),
            'name'   : '%s' % (section[1].strip()),
            'address': int(section[2],16),
            'offset' : int(section[3],16),
            'size'   : int(section[4],16)
            }

    sections_yaml.append(section_yaml)

#
# Segments
#
segment_regexp  = re.compile(elf64_regex["segments"], re.MULTILINE)
segments        = re.findall(segment_regexp, data)
segments_yaml   = []
for segment in segments:
    segment_yaml = {
            'offset'  : int(segment[1], 16),
            'vAddress': int(segment[2], 16),
            'pAddress': int(segment[3], 16),
            'fSize'   : int(segment[4], 16),
            'vSize'   : int(segment[5], 16)
            }
    segments_yaml.append(segment_yaml)

#
# Relocations
#
#relocations_regexp  = re.compile(ur'^([0-9a-fA-F]+)\s+([0-9a-fA-F]+)\s+(.\S+)\s+([0-9a-fA-F]+)\s+(.\S+)', re.MULTILINE)
#relocations         = re.findall(relocations_regexp, data)
#relocations_yaml    = []
#for relocation in relocations:
#    relocation_yaml = {
#        'offset': int(relocation[0], 16),
#        'info'  : int(relocation[1], 16),
#        'name'  : relocation[4]
#    }
#    if relocation_yaml not in relocations_yaml:
#        relocations_yaml.append(relocation_yaml);

#
# Dynamic symboles
#
extract_regexp = re.compile(elf64_regex["dynamicSymGlob"], re.MULTILINE)
dynsyms_yaml   = []
if len(re.findall(extract_regexp, data)) > 0:
    extracted      = re.findall(extract_regexp, data)[0]
    dynsyms_regexp = re.compile(elf64_regex["dynamicSymLoc"], re.MULTILINE)
    dynsyms        = re.findall(dynsyms_regexp, extracted)
    dynsyms_yaml   = []
    for dynsym in dynsyms:
        dynsym_yaml = {
            'num' : int(dynsym[0]),
            'name': dynsym[1]
        }
        dynsyms_yaml.append(dynsym_yaml)

#
# Static symbols
#
extract_regexp = re.compile(elf64_regex["staticSymGlob"], re.MULTILINE)
staticsyms = []
extracted = ""

if len(re.findall(extract_regexp, data)) > 0:
    extracted         = re.findall(extract_regexp, data)[0]
    staticsyms_regexp = re.compile(elf64_regex["staticSymLoc"], re.MULTILINE)
    staticsyms        = re.findall(staticsyms_regexp, extracted)

staticsyms_yaml = []

for staticsym in staticsyms:
    staticsym_yaml = {
        'num' : int(staticsym[0]),
        'name': staticsym[1]
    }
    staticsyms_yaml.append(staticsym_yaml)


#
# Dynamic Relocations
#
extract_regexp       = re.compile(elf64_regex["dynRelocaGlob"], re.MULTILINE)
relocations_dyn_yaml = []
if len(re.findall(extract_regexp, data)) > 0:
    extracted          = re.findall(extract_regexp, data)[0]
    regexp             = re.compile(elf64_regex["dynRelocaLoc"], re.MULTILINE)
    relocations        = re.findall(regexp, extracted)
    relocations_dyn_yaml   = []
    for reloc in relocations:
        relocation_yaml = {
            'offset' : int(reloc[0], 16),
            'info'   : int(reloc[1], 16),
            'value'  : int(reloc[2], 16),
            'name'   : reloc[3]
        }
        relocations_dyn_yaml.append(relocation_yaml)

#
# .plt.got relocations
#
extract_regexp     = re.compile(elf64_regex["pltRelocaGlob"], re.MULTILINE)
extracted          = re.findall(extract_regexp, data)[0]
regexp             = re.compile(elf64_regex["pltRelocaLoc"], re.MULTILINE)
relocations        = re.findall(regexp, extracted)
relocations_plt_yaml   = []
for reloc in relocations:
    relocation_yaml = {
        'offset' : int(reloc[0], 16),
        'info'   : int(reloc[1], 16),
        'value'  : int(reloc[2], 16),
        'name'   : reloc[3]
    }
    relocations_plt_yaml.append(relocation_yaml)

binary['Header']          = header
binary['Sections']        = sections_yaml
binary['Segments']        = segments_yaml
#binary['Relocations']     = relocations_yaml
if len(relocations_plt_yaml) > 0:
    binary['PltGotReloc']     = relocations_plt_yaml

if len(dynsyms_yaml) > 0:
    binary['DynamicSymbols'] = dynsyms_yaml

if len(relocations_dyn_yaml) > 0:
    binary['DynamicReloc'] = relocations_dyn_yaml

if len(staticsyms_yaml) > 0:
    binary['StaticSymbols'] = staticsyms_yaml

output = open(binary["filename"] + ".yaml", "w")
yaml.dump(binary, stream=output)
output.close()



#!/usr/bin/env python
import unittest
import lief
import logging
import pprint
import json
import os

from lief import Logger
Logger.set_level(lief.LOGGING_LEVEL.FATAL)
#Logger.set_level(lief.LOGGING_LEVEL.DEBUG)

from unittest import TestCase
from utils import get_sample

CURRENT_DIR = os.path.abspath(os.path.dirname(__file__))

class TestOAT64(TestCase):

    def setUp(self):
        self.logger = logging.getLogger(__name__)

    def test_multidex(self):
        WallpaperCropper2 = lief.parse(get_sample("OAT/OAT_064_AArch64_WallpaperCropper2.oat"))

        self.assertEqual(len(WallpaperCropper2.dex_files), 3)

    def test_header(self):
        WallpaperCropper2 = lief.parse(get_sample("OAT/OAT_064_AArch64_WallpaperCropper2.oat"))
        header = WallpaperCropper2.header

        self.assertEqual(header.magic, [111, 97, 116, 10])
        self.assertEqual(header.version, 64)
        self.assertEqual(header.checksum, 3369241059)
        self.assertEqual(header.instruction_set, lief.OAT.INSTRUCTION_SETS.ARM_64)
        self.assertEqual(header.nb_dex_files, 3)

        self.assertEqual(header.oat_dex_files_offset, 0)

        self.assertEqual(header.executable_offset, 3846144)
        self.assertEqual(header.i2i_bridge_offset, 0)
        self.assertEqual(header.i2c_code_bridge_offset, 0)
        self.assertEqual(header.jni_dlsym_lookup_offset, 0)
        self.assertEqual(header.quick_generic_jni_trampoline_offset, 0)
        self.assertEqual(header.quick_imt_conflict_trampoline_offset, 0)
        self.assertEqual(header.quick_resolution_trampoline_offset, 0)
        self.assertEqual(header.quick_to_interpreter_bridge_offset, 0)

        self.assertEqual(header.image_patch_delta, 0)

        self.assertEqual(header.image_file_location_oat_checksum, 285056181)
        self.assertEqual(header.image_file_location_oat_data_begin, 1897058304)

    def test_dex_files(self):
        WallpaperCropper2 = lief.parse(get_sample("OAT/OAT_064_AArch64_WallpaperCropper2.oat"))
        dex_files = WallpaperCropper2.dex_files

        self.assertEqual(len(dex_files), WallpaperCropper2.header.nb_dex_files)

        # Dex File 0
        dex = dex_files[0]
        self.assertEqual(dex.name, "classes.dex")
        self.assertEqual(dex.location, "/system/priv-app/WallpaperCropper2/WallpaperCropper2.apk")
        self.assertEqual(len(dex.raw(deoptimize=False)), dex.header.file_size)

        # Dex File 1
        dex = dex_files[1]
        #self.assertEqual(dex.name, "classes2.dex")
        self.assertEqual(dex.location, "/system/priv-app/WallpaperCropper2/WallpaperCropper2.apk:classes2.dex")
        self.assertEqual(len(dex.raw(deoptimize=False)), dex.header.file_size)

        # Dex File 2
        dex = dex_files[2]
        #self.assertEqual(dex.name, "classes3.dex")
        self.assertEqual(dex.location, "/system/priv-app/WallpaperCropper2/WallpaperCropper2.apk:classes3.dex")
        self.assertEqual(len(dex.raw(deoptimize=False)), dex.header.file_size)


    def test_oat_dex_files(self):
        WallpaperCropper2 = lief.parse(get_sample("OAT/OAT_064_AArch64_WallpaperCropper2.oat"))
        self.assertEqual(len(WallpaperCropper2.oat_dex_files), 3)

        # OAT Dex File 0
        oat_dex_file = WallpaperCropper2.oat_dex_files[0]

        self.assertEqual(oat_dex_file.location, "/system/priv-app/WallpaperCropper2/WallpaperCropper2.apk")
        self.assertEqual(oat_dex_file.checksum, 0xbb07e4e)
        self.assertEqual(oat_dex_file.dex_offset, 0x23a4)
        self.assertTrue(oat_dex_file.has_dex_file)

        # OAT Dex File 1
        oat_dex_file = WallpaperCropper2.oat_dex_files[1]

        self.assertEqual(oat_dex_file.location, "/system/priv-app/WallpaperCropper2/WallpaperCropper2.apk:classes2.dex")
        self.assertEqual(oat_dex_file.checksum, 1150225935)
        self.assertEqual(oat_dex_file.dex_offset, 340324)
        self.assertTrue(oat_dex_file.has_dex_file)

        # OAT Dex File 2
        oat_dex_file = WallpaperCropper2.oat_dex_files[2]

        self.assertEqual(oat_dex_file.location, "/system/priv-app/WallpaperCropper2/WallpaperCropper2.apk:classes3.dex")
        self.assertEqual(oat_dex_file.checksum, 459332982)
        self.assertEqual(oat_dex_file.dex_offset, 1617040)
        self.assertTrue(oat_dex_file.has_dex_file)

    def test_oat_classes(self):
        WallpaperCropper2 = lief.parse(get_sample("OAT/OAT_064_AArch64_WallpaperCropper2.oat"))
        self.assertEqual(len(WallpaperCropper2.classes), 1992)

        # OAT Class 0
        cls = WallpaperCropper2.get_class("android.support.v4.widget.ViewDragHelper")

        self.assertEqual(cls.fullname, "Landroid/support/v4/widget/ViewDragHelper;")
        self.assertEqual(cls.index, 1066)
        self.assertEqual(len(cls.methods), 49)
        self.assertEqual(cls.status, lief.OAT.OAT_CLASS_STATUS.VERIFIED)
        self.assertEqual(cls.type, lief.OAT.OAT_CLASS_TYPES.SOME_COMPILED)

        # OAT Class 1
        cls = WallpaperCropper2.get_class("com.android.keyguard.KeyguardTransportControlView$SavedState$1")

        self.assertEqual(cls.fullname, "Lcom/android/keyguard/KeyguardTransportControlView$SavedState$1;")
        self.assertEqual(cls.index, 207)
        self.assertEqual(len(cls.methods), 5)
        self.assertEqual(cls.status, lief.OAT.OAT_CLASS_STATUS.INITIALIZED)
        self.assertEqual(cls.type, lief.OAT.OAT_CLASS_TYPES.ALL_COMPILED)

        # OAT Class 2
        cls = WallpaperCropper2.get_class("android.support.v4.os.ParcelableCompatCreatorHoneycombMR2Stub")

        self.assertEqual(cls.fullname, "Landroid/support/v4/os/ParcelableCompatCreatorHoneycombMR2Stub;")
        self.assertEqual(cls.index, 566)
        self.assertEqual(len(cls.methods), 2)
        self.assertEqual(cls.status, lief.OAT.OAT_CLASS_STATUS.INITIALIZED)
        self.assertEqual(cls.type, lief.OAT.OAT_CLASS_TYPES.ALL_COMPILED)

        s = sum(len(cls.methods) for cls in WallpaperCropper2.classes)
        self.assertEqual(len(WallpaperCropper2.methods), s)

        # dex_sum = sum(len(dex.classes) for dex in WallpaperCropper2.dex_files)
        # self.assertEqual(len(WallpaperCropper2.classes), dex_sum)

    def test_oat_methods(self):
        WallpaperCropper2 = lief.parse(get_sample("OAT/OAT_064_AArch64_WallpaperCropper2.oat"))
        self.assertEqual(len(WallpaperCropper2.methods), 13830)

        self.assertTrue(all(m.is_compiled for m in WallpaperCropper2.methods))

        # OAT Method 0
        # ============
        method = WallpaperCropper2.methods[0]
        self.assertEqual(method.name, "<init>")
        self.assertEqual(method.oat_class, WallpaperCropper2.get_class("com/android/gallery3d/ds/DsWallpaperSetting"))


        # OAT Method 100
        # ==============
        method = WallpaperCropper2.methods[100]
        self.assertEqual(method.name, "deleteTag")
        self.assertEqual(method.oat_class, WallpaperCropper2.get_class("com/android/gallery3d/exif/ExifInterface"))


class TestOAT79(TestCase):

    def setUp(self):
        self.logger = logging.getLogger(__name__)

    def test_multidex(self):
        hangout = lief.parse(get_sample("OAT/OAT_079_AArch64_Hangouts.oat"))

        self.assertEqual(len(hangout.dex_files), 2)

    def test_header(self):
        pm = lief.parse(get_sample("OAT/OAT_079_AArch64_pm.oat"))
        header = pm.header

        self.assertEqual(header.magic, [111, 97, 116, 10])
        self.assertEqual(header.version, 79)
        self.assertEqual(header.checksum, 2466303069)
        self.assertEqual(header.instruction_set, lief.OAT.INSTRUCTION_SETS.ARM_64)
        self.assertEqual(header.nb_dex_files, 1)

        self.assertEqual(header.oat_dex_files_offset, 0)

        self.assertEqual(header.executable_offset, 73728)
        self.assertEqual(header.i2i_bridge_offset, 0)
        self.assertEqual(header.i2c_code_bridge_offset, 0)
        self.assertEqual(header.jni_dlsym_lookup_offset, 0)
        self.assertEqual(header.quick_generic_jni_trampoline_offset, 0)
        self.assertEqual(header.quick_imt_conflict_trampoline_offset, 0)
        self.assertEqual(header.quick_resolution_trampoline_offset, 0)
        self.assertEqual(header.quick_to_interpreter_bridge_offset, 0)

        self.assertEqual(header.image_patch_delta, 0)

        self.assertEqual(header.image_file_location_oat_checksum, 3334846204)
        self.assertEqual(header.image_file_location_oat_data_begin, 1893416960)

    def test_decompile(self):
        calldeviceid = lief.parse(get_sample('OAT/OAT_079_x86-64_CallDeviceId.oat'))

        self.assertEqual(len(calldeviceid.dex_files), 1)

        dex2dex_json_info_lhs = json.loads(calldeviceid.dex2dex_json_info)
        dex2dex_json_info_rhs = {'classes.dex': {'Lre/android/art/CallDeviceId;': {'3': {'0': 0}}}}
        self.assertEqual(dex2dex_json_info_lhs, dex2dex_json_info_rhs)

    def test_dex_files(self):
        CallDeviceId = lief.parse(get_sample("OAT/OAT_079_x86-64_CallDeviceId.oat"))
        dex_files = CallDeviceId.dex_files

        self.assertEqual(len(dex_files), CallDeviceId.header.nb_dex_files)

        # Dex File 0
        dex = dex_files[0]
        self.assertEqual(dex.name, "classes.dex")
        self.assertEqual(dex.location, "/data/local/tmp/CallDeviceId.dex")
        self.assertEqual(len(dex.raw(deoptimize=False)), dex.header.file_size)



    def test_oat_dex_files(self):
        CallDeviceId = lief.parse(get_sample("OAT/OAT_079_x86-64_CallDeviceId.oat"))
        self.assertEqual(len(CallDeviceId.oat_dex_files), 1)

        # OAT Dex File 0
        oat_dex_file = CallDeviceId.oat_dex_files[0]

        self.assertEqual(oat_dex_file.location, "/data/local/tmp/CallDeviceId.dex")
        self.assertEqual(oat_dex_file.checksum, 284645792)
        self.assertEqual(oat_dex_file.dex_offset, 1320)
        self.assertTrue(oat_dex_file.has_dex_file)

    def test_oat_classes(self):
        CallDeviceId = lief.parse(get_sample("OAT/OAT_079_x86-64_CallDeviceId.oat"))
        self.assertEqual(len(CallDeviceId.classes), 1)

        # OAT Class 0
        cls = CallDeviceId.classes[0]

        self.assertEqual(cls.fullname, "Lre/android/art/CallDeviceId;")
        self.assertEqual(cls.index, 0)
        self.assertEqual(len(cls.methods), 1)
        self.assertEqual(cls.status, lief.OAT.OAT_CLASS_STATUS.INITIALIZED)
        self.assertEqual(cls.type, lief.OAT.OAT_CLASS_TYPES.SOME_COMPILED)


    def test_oat_methods(self):
        CallDeviceId = lief.parse(get_sample("OAT/OAT_079_x86-64_CallDeviceId.oat"))
        self.assertEqual(len(CallDeviceId.methods), 1)

        self.assertTrue(all(m.is_dex2dex_optimized for m in CallDeviceId.methods))

        # OAT Method 0
        # ============
        method = CallDeviceId.methods[0]
        self.assertEqual(method.name, "getIMEI")
        self.assertEqual(method.oat_class, CallDeviceId.get_class("Lre/android/art/CallDeviceId;"))

class TestOAT124(TestCase):

    def setUp(self):
        self.logger = logging.getLogger(__name__)

    def test_header(self):
        emode = lief.parse(get_sample("OAT/OAT_124_AArch64_EngineeringMode.oat"))
        header = emode.header

        self.assertEqual(header.magic, [111, 97, 116, 10])
        self.assertEqual(header.version, 124)
        self.assertEqual(header.checksum, 2299270308)
        self.assertEqual(header.instruction_set, lief.OAT.INSTRUCTION_SETS.ARM_64)
        self.assertEqual(header.nb_dex_files, 1)

        self.assertEqual(header.oat_dex_files_offset, 0)

        self.assertEqual(header.executable_offset, 65536)
        self.assertEqual(header.i2i_bridge_offset, 0)
        self.assertEqual(header.i2c_code_bridge_offset, 0)
        self.assertEqual(header.jni_dlsym_lookup_offset, 0)
        self.assertEqual(header.quick_generic_jni_trampoline_offset, 0)
        self.assertEqual(header.quick_imt_conflict_trampoline_offset, 0)
        self.assertEqual(header.quick_resolution_trampoline_offset, 0)
        self.assertEqual(header.quick_to_interpreter_bridge_offset, 0)

        self.assertEqual(header.image_patch_delta, 0)

        self.assertEqual(header.image_file_location_oat_checksum, 1759409278)
        self.assertEqual(header.image_file_location_oat_data_begin, 1893093376)

    def test_oat_dex_files(self):
        CallDeviceId = lief.parse(get_sample("OAT/OAT_124_x86-64_CallDeviceId.oat"))
        self.assertEqual(len(CallDeviceId.oat_dex_files), 1)

        # OAT Dex File 0
        oat_dex_file = CallDeviceId.oat_dex_files[0]

        self.assertEqual(oat_dex_file.location, "/data/local/tmp/CallDeviceId.dex")
        self.assertEqual(oat_dex_file.checksum, 284645792)
        self.assertEqual(oat_dex_file.dex_offset, 28)
        self.assertFalse(oat_dex_file.has_dex_file)

    def test_oat_classes(self):
        oat_file  = get_sample("OAT/OAT_124_x86-64_CallDeviceId.oat")
        vdex_file = get_sample("VDEX/VDEX_06_x86-64_CallDeviceId.vdex")

        CallDeviceId = lief.OAT.parse(oat_file, vdex_file)
        self.assertEqual(len(CallDeviceId.classes), 1)

        # OAT Class 0
        cls = CallDeviceId.classes[0]

        self.assertEqual(cls.fullname, "Lre/android/art/CallDeviceId;")
        self.assertEqual(cls.index, 0)
        self.assertEqual(len(cls.methods), 1)
        self.assertEqual(cls.status, lief.OAT.OAT_CLASS_STATUS.INITIALIZED)
        self.assertEqual(cls.type, lief.OAT.OAT_CLASS_TYPES.SOME_COMPILED)


    def test_oat_methods(self):
        oat_file  = get_sample("OAT/OAT_124_x86-64_CallDeviceId.oat")
        vdex_file = get_sample("VDEX/VDEX_06_x86-64_CallDeviceId.vdex")

        CallDeviceId = lief.OAT.parse(oat_file, vdex_file)
        self.assertEqual(len(CallDeviceId.methods), 1)

        self.assertTrue(all(m.is_dex2dex_optimized for m in CallDeviceId.methods))

        # OAT Method 0
        # ============
        method = CallDeviceId.methods[0]
        self.assertEqual(method.name, "getIMEI")
        self.assertEqual(method.oat_class, CallDeviceId.get_class("Lre/android/art/CallDeviceId;"))


class TestOAT131(TestCase):

    def setUp(self):
        self.logger = logging.getLogger(__name__)

    def test_header(self):
        CallDeviceId = lief.parse(get_sample("OAT/OAT_131_x86_CallDeviceId.oat"))
        header = CallDeviceId.header

        self.assertEqual(header.magic, [111, 97, 116, 10])
        self.assertEqual(header.version, 131)
        self.assertEqual(header.checksum, 0x8e82f9b5)
        self.assertEqual(header.instruction_set, lief.OAT.INSTRUCTION_SETS.X86)
        self.assertEqual(header.nb_dex_files, 1)

        self.assertEqual(header.oat_dex_files_offset, 1484)

        self.assertEqual(header.executable_offset, 0x1000)
        self.assertEqual(header.i2i_bridge_offset, 0)
        self.assertEqual(header.i2c_code_bridge_offset, 0)
        self.assertEqual(header.jni_dlsym_lookup_offset, 0)
        self.assertEqual(header.quick_generic_jni_trampoline_offset, 0)
        self.assertEqual(header.quick_imt_conflict_trampoline_offset, 0)
        self.assertEqual(header.quick_resolution_trampoline_offset, 0)
        self.assertEqual(header.quick_to_interpreter_bridge_offset, 0)

        self.assertEqual(header.image_patch_delta, 15335424)

        self.assertEqual(header.image_file_location_oat_checksum, 0xdacfe293)
        self.assertEqual(header.image_file_location_oat_data_begin, 0x716a9000)

    def test_oat_dex_files(self):
        CallDeviceId = lief.parse(get_sample("OAT/OAT_131_x86_CallDeviceId.oat"))
        self.assertEqual(len(CallDeviceId.oat_dex_files), 1)

        # OAT Dex File 0
        oat_dex_file = CallDeviceId.oat_dex_files[0]

        self.assertEqual(oat_dex_file.location, "/data/local/tmp/CallDeviceId.dex")
        self.assertEqual(oat_dex_file.checksum, 284645792)
        self.assertEqual(oat_dex_file.dex_offset, 28)
        self.assertFalse(oat_dex_file.has_dex_file)

    def test_oat_classes(self):
        oat_file  = get_sample("OAT/OAT_131_x86_CallDeviceId.oat")
        vdex_file = get_sample("VDEX/VDEX_10_x86_CallDeviceId.vdex")

        #oat_file  = get_sample("OAT/OAT_131_AArch64_svc.oat")
        #vdex_file = get_sample("VDEX/VDEX_10_AArch64_svc.vdex")

        CallDeviceId = lief.OAT.parse(oat_file, vdex_file)
        self.assertEqual(len(CallDeviceId.classes), 1)

        # OAT Class 0
        cls = CallDeviceId.classes[0]

        self.assertEqual(cls.fullname, "Lre/android/art/CallDeviceId;")
        self.assertEqual(cls.index, 0)
        self.assertEqual(len(cls.methods), 0)
        #self.assertEqual(cls.status, lief.OAT.OAT_CLASS_STATUS.INITIALIZED) # TODO
        self.assertEqual(cls.type, lief.OAT.OAT_CLASS_TYPES.NONE_COMPILED)

    def test_oat_methods(self):
        oat_file  = get_sample("OAT/OAT_124_x86-64_CallDeviceId.oat")
        vdex_file = get_sample("VDEX/VDEX_06_x86-64_CallDeviceId.vdex")

        CallDeviceId = lief.OAT.parse(oat_file, vdex_file)
        self.assertEqual(len(CallDeviceId.methods), 1)

        self.assertTrue(all(m.is_dex2dex_optimized for m in CallDeviceId.methods))

        # OAT Method 0
        # ============
        method = CallDeviceId.methods[0]
        self.assertEqual(method.name, "getIMEI")
        self.assertEqual(method.oat_class, CallDeviceId.get_class("Lre/android/art/CallDeviceId;"))


class TestOAT(TestCase):

    def setUp(self):
        self.logger = logging.getLogger(__name__)

    def test_header_key_values(self):
        CallDeviceId = lief.parse(get_sample('OAT/OAT_079_x86-64_CallDeviceId.oat'))
        header = CallDeviceId.header

        self.assertEqual(header[lief.OAT.HEADER_KEYS.IMAGE_LOCATION],
                "/data/dalvik-cache/x86_64/system@framework@boot.art:"
                "/data/dalvik-cache/x86_64/system@framework@boot-core-libart.art:/data/dalvik-cache/x86_64/system@framework@boot-conscrypt.art:"
                "/data/dalvik-cache/x86_64/system@framework@boot-okhttp.art:/data/dalvik-cache/x86_64/system@framework@boot-core-junit.art:"
                "/data/dalvik-cache/x86_64/system@framework@boot-bouncycastle.art:/data/dalvik-cache/x86_64/system@framework@boot-ext.art:"
                "/data/dalvik-cache/x86_64/system@framework@boot-framework.art:/data/dalvik-cache/x86_64/system@framework@boot-telephony-common.art:"
                "/data/dalvik-cache/x86_64/system@framework@boot-voip-common.art:/data/dalvik-cache/x86_64/system@framework@boot-ims-common.art:"
                "/data/dalvik-cache/x86_64/system@framework@boot-apache-xml.art:/data/dalvik-cache/x86_64/system@framework@boot-org.apache.http.legacy.boot.art")

        self.assertEqual(header[lief.OAT.HEADER_KEYS.DEX2OAT_CMD_LINE],
                "--dex-file=/data/local/tmp/CallDeviceId.dex --oat-file=/data/local/tmp/CallDeviceId.oat "
                "--boot-image=/system/framework/boot.art --instruction-set=x86_64 "
                "--compiler-filter=interpret-only --compiler-backend=Quick")

        self.assertEqual(header[lief.OAT.HEADER_KEYS.PIC], "false")
        self.assertEqual(header[lief.OAT.HEADER_KEYS.HAS_PATCH_INFO], "false")
        self.assertEqual(header[lief.OAT.HEADER_KEYS.DEBUGGABLE], "false")
        self.assertEqual(header[lief.OAT.HEADER_KEYS.NATIVE_DEBUGGABLE], "false")
        self.assertEqual(header[lief.OAT.HEADER_KEYS.COMPILER_FILTER], "interpret-only")

        header[lief.OAT.HEADER_KEYS.DEBUGGABLE] = "true"
        self.assertEqual(header[lief.OAT.HEADER_KEYS.DEBUGGABLE], "true")

        self.assertEqual(len(header.keys), 8)

        for e in header.key_values:
            self.assertEqual(header[e.key], e.value)

        for x in header.key_values:
            x.value = "foo"

        self.assertTrue(all(k == "foo" for k in header.values))

if __name__ == '__main__':

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    root_logger.addHandler(ch)

    unittest.main(verbosity=2)




#!/usr/bin/env python
import lief
import sys

binary = lief.parse(sys.argv[1])
builder = lief.PE.Builder(binary)
builder.build_imports(True)
builder.patch_imports(True)
builder.build_relocations(False)
builder.build_tls(False)
builder.build_resources(False)
builder.build();

builder.write(sys.argv[2])



#!/usr/bin/env python
import unittest
import logging
import os
import sys
import stat
import re
import subprocess
import tempfile
import shutil
from subprocess import Popen

import lief
from lief.ELF import Section

from unittest import TestCase
from utils import get_sample

CURRENT_DIRECTORY = os.path.dirname(os.path.abspath(__file__))
STUB = lief.parse(os.path.join(CURRENT_DIRECTORY, "hello_lief.bin"))

class TestAddSection(TestCase):
    def setUp(self):
        self.logger = logging.getLogger(__name__)
        self.tmp_dir = tempfile.mkdtemp(suffix='_lief_test_section')
        self.logger.debug("temp dir: {}".format(self.tmp_dir))


    @unittest.skipUnless(sys.platform.startswith("linux"), "requires Linux")
    def test_simple(self):
        sample_path = get_sample('ELF/ELF64_x86-64_binary_ls.bin')
        output      = os.path.join(self.tmp_dir, "ls.section")

        ls = lief.parse(sample_path)
        for i in range(10):
            section = Section(".test.{:d}".format(i), lief.ELF.SECTION_TYPES.PROGBITS)
            section += lief.ELF.SECTION_FLAGS.EXECINSTR
            section += lief.ELF.SECTION_FLAGS.WRITE
            section.content   = STUB.segments[0].content # First LOAD segment which holds payload
            if i % 2 == 0:
                section = ls.add(section, loaded=True)
                ls.header.entrypoint = section.virtual_address + STUB.header.entrypoint
            else:
                section = ls.add(section, loaded=False)

        ls.write(output)

        st = os.stat(output)
        os.chmod(output, st.st_mode | stat.S_IEXEC)

        p = Popen(output, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        stdout, _ = p.communicate()
        self.logger.debug(stdout.decode("utf8"))
        self.assertIsNotNone(re.search(r'LIEF is Working', stdout.decode("utf8")))


    @unittest.skipUnless(sys.platform.startswith("linux"), "requires Linux")
    def test_gcc(self):
        sample_path = get_sample('ELF/ELF64_x86-64_binary_gcc.bin')
        output      = os.path.join(self.tmp_dir, "gcc.section")

        gcc = lief.parse(sample_path)
        for i in range(10):
            section = Section(".test.{:d}".format(i), lief.ELF.SECTION_TYPES.PROGBITS)
            section.type     = lief.ELF.SECTION_TYPES.PROGBITS
            section         += lief.ELF.SECTION_FLAGS.EXECINSTR
            section         += lief.ELF.SECTION_FLAGS.WRITE
            section.content  = STUB.segments[0].content # First LOAD segment which holds payload

            if i % 2 == 0:
                section = gcc.add(section, loaded=True)
                gcc.header.entrypoint = section.virtual_address + STUB.header.entrypoint
            else:
                section = gcc.add(section, loaded=False)

        gcc.write(output)

        st = os.stat(output)
        os.chmod(output, st.st_mode | stat.S_IEXEC)

        p = Popen(output, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        stdout, _ = p.communicate()
        self.logger.debug(stdout.decode("utf8"))
        self.assertIsNotNone(re.search(r'LIEF is Working', stdout.decode("utf8")))


    def tearDown(self):
        # Delete it
        if os.path.isdir(self.tmp_dir):
            shutil.rmtree(self.tmp_dir)

if __name__ == '__main__':

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    root_logger.addHandler(ch)

    unittest.main(verbosity=2)




#!/usr/bin/env python
import unittest
import lief
import logging
from io import open

from lief import Logger
Logger.set_level(lief.LOGGING_LEVEL.WARNING)

from unittest import TestCase
from utils import get_sample

class TestPythonApi(TestCase):

    def setUp(self):
        self.logger = logging.getLogger(__name__)

    def test_io(self):
        lspath = get_sample('ELF/ELF64_x86-64_binary_ls.bin')

        with open(lspath, 'r') as f:
            ls = lief.parse(f);
            self.assertIsNotNone(ls.abstract.header)


        with open(lspath, 'rb') as f:
            ls = lief.parse(f);
            self.assertIsNotNone(ls.abstract.header)


        with open(lspath, 'rb') as f:
            ls = lief.ELF.parse(f);
            self.assertIsNotNone(ls.abstract.header)

        with open(get_sample('PE/PE64_x86-64_binary_HelloWorld.exe'), 'rb') as f:
            binary = lief.PE.parse(f);
            self.assertIsNotNone(binary.abstract.header)

        with open(get_sample('MachO/MachO64_x86-64_binary_dd.bin'), 'rb') as f:
            binary = lief.MachO.parse(f)[0];
            self.assertIsNotNone(binary.abstract.header)



if __name__ == '__main__':

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    root_logger.addHandler(ch)

    unittest.main(verbosity=2)




import unittest
import logging
import shlex
import subprocess
import tempfile
import lief
import os
import stat
import shutil
import sys
from collections import namedtuple

class CommandResult(object):
    def __init__(self, output, error, retcode, process=None):
        self.output = output
        self.error = error
        self.retcode = retcode
        self.process = process

    def __bool__(self):
        return not self.retcode

    def __str__(self):
        if bool(self):
            return self.output
        return self.error




lief.Logger.set_level(lief.LOGGING_LEVEL.INFO)

LOGGING_LEVEL = logging.DEBUG

LIBADD = """\
#include <stdlib.h>
#include <stdio.h>
#define LOCAL __attribute__ ((visibility ("hidden")))

LOCAL int add_hidden(int a, int b) {
  printf("[LOCAL] %d + %d = %d\\n", a, b, a+b);
  return a + b;
}

int main(int argc, char** argv) {

  if (argc != 3) {
    printf("Usage: %s <a> <b>\\n", argv[0]);
    exit(-1);
  }

  printf("Hello\\n");
  int res = add_hidden(atoi(argv[1]), atoi(argv[2]));
  printf("From add_hidden@libadd.so a + b = %d\\n", res);
  return 0;
}
"""


BINADD = """\
#include <stdio.h>
#include <stdlib.h>
extern int add_hidden(int a, int b);

int main(int argc, char **argv) {
  if (argc != 3) {
    printf("Usage: %s <a> <b>\\n", argv[0]);
    exit(-1);
  }

  printf("Hello\\n");
  int res = add_hidden(atoi(argv[1]), atoi(argv[2]));
  printf("From add_hidden@libadd.so a + b = %d\\n", res);
  return 0;
}
"""


class TestBin2Lib(unittest.TestCase):
    LOGGER = logging.getLogger(__name__)

    def setUp(self):
        self._logger = logging.getLogger(__name__)

    @staticmethod
    def run_cmd(cmd):
        TestBin2Lib.LOGGER.debug("Running: '{}'".format(cmd))
        cmd = shlex.split(cmd)
        p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
        stdout, stderr = p.communicate()

        if stdout:
            TestBin2Lib.LOGGER.debug(stdout)

        if stderr:
            TestBin2Lib.LOGGER.error(stderr)

        return CommandResult(stdout, stderr, p.returncode)

    @unittest.skipUnless(sys.platform.startswith("linux"), "requires Linux")
    def test_libadd(self):


        _, binaddc = tempfile.mkstemp(prefix="binadd_", suffix=".c")
        _, libaddc = tempfile.mkstemp(prefix="libadd_", suffix=".c")

        self._logger.debug(binaddc)
        self._logger.debug(libaddc)

        fd, binadd  = tempfile.mkstemp(prefix="binadd_", suffix=".bin")
        _, libadd  = tempfile.mkstemp(prefix="libadd_", suffix=".so")
        _, libadd2 = tempfile.mkstemp(prefix="libadd2_", suffix=".so")

        self._logger.debug(binadd)
        self._logger.debug(libadd)
        self._logger.debug(libadd2)


        with open(binaddc, 'w') as f:
            f.write(BINADD)

        with open(libaddc, 'w') as f:
            f.write(LIBADD)

        #compiler = os.getenv("CC", "cc")
        #compiler = shutil.which(compiler)
        compiler = "/usr/bin/cc"

        # Compile libadd
        r = self.run_cmd("{compiler} -mcmodel=large -fPIE -pie -o {output} {input}".format(
            compiler=compiler,
            output=libadd,
            input=libaddc))
        self.assertTrue(r, msg="Unable to compile libadd")

        libadd = lief.parse(libadd)

        libadd_hidden            = libadd.get_symbol("add_hidden")
        libadd_hidden.binding    = lief.ELF.SYMBOL_BINDINGS.GLOBAL
        libadd_hidden.visibility = lief.ELF.SYMBOL_VISIBILITY.DEFAULT
        libadd_hidden            = libadd.add_dynamic_symbol(libadd_hidden, lief.ELF.SymbolVersion.global_)
        self._logger.debug(libadd_hidden)

        libadd.add(lief.ELF.DynamicSharedObject(os.path.basename(libadd2)))

        libadd.write(libadd2)

        lib_directory = os.path.dirname(libadd2)
        libname = os.path.basename(libadd2)[3:-3] # libadd.so ---> add

        r = self.run_cmd("{compiler} -mcmodel=large -fPIE -pie -Wl,-rpath={libdir} -L{libdir} -o {output} {input} -l{libadd2}".format(
            compiler=compiler,
            libdir=lib_directory,
            libadd2=libname,
            output=binadd,
            input=binaddc))
        self.assertTrue(r, msg="Unable to compile binadd")

        os.close(fd)
        st = os.stat(binadd)
        os.chmod(binadd, st.st_mode | stat.S_IEXEC)

        r = self.run_cmd(binadd + " 1 2")
        self.assertTrue(r)
        self.assertIn("From add_hidden@libadd.so a + b = 3", r.output)


    @unittest.skipUnless(sys.platform.startswith("linux"), "requires Linux")
    def test_libadd_api(self):
        _, binaddc = tempfile.mkstemp(prefix="binadd_", suffix=".c")
        _, libaddc = tempfile.mkstemp(prefix="libadd_", suffix=".c")

        self._logger.debug(binaddc)
        self._logger.debug(libaddc)

        fd, binadd  = tempfile.mkstemp(prefix="binadd_", suffix=".bin")
        _, libadd  = tempfile.mkstemp(prefix="libadd_", suffix=".so")
        _, libadd2 = tempfile.mkstemp(prefix="libadd2_", suffix=".so")

        self._logger.debug(binadd)
        self._logger.debug(libadd)
        self._logger.debug(libadd2)


        with open(binaddc, 'w') as f:
            f.write(BINADD)

        with open(libaddc, 'w') as f:
            f.write(LIBADD)

        #compiler = os.getenv("CC", "cc")
        #compiler = shutil.which(compiler)
        compiler = "/usr/bin/cc"

        # Compile libadd
        r = self.run_cmd("{compiler} -mcmodel=large -fPIE -pie -o {output} {input}".format(
            compiler=compiler,
            output=libadd,
            input=libaddc))
        self.assertTrue(r, msg="Unable to compile libadd")

        libadd = lief.parse(libadd)
        libadd.export_symbol("add_hidden")
        libadd.write(libadd2)

        lib_directory = os.path.dirname(libadd2)
        libname = os.path.basename(libadd2)[3:-3] # libadd.so ---> add

        r = self.run_cmd("{compiler} -mcmodel=large -fPIE -pie -Wl,-rpath={libdir} -L{libdir} -o {output} {input} -l{libadd2}".format(
            compiler=compiler,
            libdir=lib_directory,
            libadd2=libname,
            output=binadd,
            input=binaddc))
        self.assertTrue(r, msg="Unable to compile binadd")

        os.close(fd)
        st = os.stat(binadd)
        os.chmod(binadd, st.st_mode | stat.S_IEXEC)

        r = self.run_cmd(binadd + " 1 2")
        self.assertTrue(r)
        self.assertIn("From add_hidden@libadd.so a + b = 3", r.output)


    @unittest.skipUnless(sys.platform.startswith("linux"), "requires Linux")
    def test_libadd_api2(self):
        _, binaddc = tempfile.mkstemp(prefix="binadd_", suffix=".c")
        _, libaddc = tempfile.mkstemp(prefix="libadd_", suffix=".c")

        self._logger.debug(binaddc)
        self._logger.debug(libaddc)

        fd, binadd  = tempfile.mkstemp(prefix="binadd_", suffix=".bin")
        _, libadd  = tempfile.mkstemp(prefix="libadd_", suffix=".so")
        _, libadd2 = tempfile.mkstemp(prefix="libadd2_", suffix=".so")

        self._logger.debug(binadd)
        self._logger.debug(libadd)
        self._logger.debug(libadd2)


        with open(binaddc, 'w') as f:
            f.write(BINADD)

        with open(libaddc, 'w') as f:
            f.write(LIBADD)

        #compiler = os.getenv("CC", "cc")
        #compiler = shutil.which(compiler)
        compiler = "/usr/bin/cc"

        # Compile libadd
        r = self.run_cmd("{compiler} -mcmodel=large -fPIE -pie -o {output} {input}".format(
            compiler=compiler,
            output=libadd,
            input=libaddc))
        self.assertTrue(r, msg="Unable to compile libadd")

        libadd = lief.parse(libadd)
        add_hidden_static = libadd.get_static_symbol("add_hidden")
        libadd.add_exported_function(add_hidden_static.value, add_hidden_static.name)
        libadd.write(libadd2)

        lib_directory = os.path.dirname(libadd2)
        libname = os.path.basename(libadd2)[3:-3] # libadd.so ---> add

        r = self.run_cmd("{compiler} -mcmodel=large -fPIE -pie -Wl,-rpath={libdir} -L{libdir} -o {output} {input} -l{libadd2}".format(
            compiler=compiler,
            libdir=lib_directory,
            libadd2=libname,
            output=binadd,
            input=binaddc))
        self.assertTrue(r, msg="Unable to compile binadd")

        os.close(fd)
        st = os.stat(binadd)
        os.chmod(binadd, st.st_mode | stat.S_IEXEC)

        r = self.run_cmd(binadd + " 1 2")
        self.assertTrue(r)
        self.assertIn("From add_hidden@libadd.so a + b = 3", r.output)


if __name__ == "__main__":

    root_logger = logging.getLogger()
    root_logger.addHandler(logging.StreamHandler())
    root_logger.setLevel(LOGGING_LEVEL)
    for h in root_logger.handlers:
        h.setLevel(LOGGING_LEVEL)

    unittest.main(verbosity=2)





#!/usr/bin/env python3
import unittest
import logging
import os
import stat
import sys
import re
import subprocess
import platform
import tempfile
import shutil

from subprocess import Popen

import lief

from unittest import TestCase
from utils import get_sample

LIBADD_C = """\
#include <stdlib.h>
#include <stdio.h>

int add(int a, int b);

int add(int a, int b) {
  printf("%d + %d = %d\\n", a, b, a+b);
  return a + b;
}
"""

BINADD_C = """\
#include <stdio.h>
#include <stdlib.h>

int add(int a, int b);

int main(int argc, char **argv) {
  if (argc != 3) {
    printf("Usage: %s <a> <b>\\n", argv[0]);
    exit(-1);
  }

  int res = add(atoi(argv[1]), atoi(argv[2]));
  printf("From myLIb, a + b = %d\\n", res);
  return 0;
}
"""

class TestHash(TestCase):

    def setUp(self):
        self.logger  = logging.getLogger(__name__)
        self.tmp_dir = tempfile.mkdtemp(suffix='_lief_testhash')
        self.logger.debug("temp dir: {}".format(self.tmp_dir))

        self.binadd_path = os.path.join(self.tmp_dir, "binadd.c")
        self.libadd_path = os.path.join(self.tmp_dir, "libadd.c")

        self.libadd_so  = os.path.join(self.tmp_dir, "libadd.so")
        self.binadd_bin = os.path.join(self.tmp_dir, "binadd.bin")

        self.compiler = shutil.which(os.environ.get('CC', 'cc'))
        if self.compiler is None:
            self.logger.error("Unable to find a compiler")
            sys.exit(0)

        self.logger.debug("Compiler: {}".format(self.compiler))

        with open(self.binadd_path, 'w') as f:
            f.write(BINADD_C)

        with open(self.libadd_path, 'w') as f:
            f.write(LIBADD_C)

    def compile_libadd(self, input, extra_flags=[]):
        if os.path.isfile(self.libadd_so):
            os.remove(self.libadd_so)
        CC_FLAGS = ['-fPIC', '-shared'] + extra_flags
        cmd = [self.compiler, '-o', self.libadd_so] + CC_FLAGS + [input]
        self.logger.debug("Compile 'libadd' with: {}".format(" ".join(cmd)))
        p = Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        stdout, _ = p.communicate()
        self.logger.debug(stdout)


    def compile_binadd(self, input, extra_flags=[]):

        if os.path.isfile(self.binadd_bin):
            os.remove(self.binadd_bin)

        CC_FLAGS = ['-L', self.tmp_dir] + extra_flags
        cmd = [self.compiler, '-o', self.binadd_bin] + CC_FLAGS + [input, '-ladd']
        self.logger.debug("Compile 'binadd' with: {}".format(" ".join(cmd)))
        p = Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        stdout, _ = p.communicate()
        self.logger.debug(stdout)

    def obfuscate(self):

        libadd = lief.parse(self.libadd_so)
        binadd = lief.parse(self.binadd_bin)

        libadd_dynsym = libadd.dynamic_symbols
        binadd_dynsym = binadd.dynamic_symbols

        # Change add in the libary
        for sym in libadd_dynsym:
            if sym.name == "add":
                sym.name = "abc"

        # Change "add" in the binary
        for sym in binadd_dynsym:
            if sym.name == "add":
                sym.name = "abc"


        # change library name in the binary
        for entry in binadd.dynamic_entries:
            if entry.tag == lief.ELF.DYNAMIC_TAGS.NEEDED and entry.name == "libadd.so":
                entry.name = "libabc.so"

        libadd_modified = os.path.join(self.tmp_dir, "libabc.so")
        binadd_modified = os.path.join(self.tmp_dir, "binadd_obf.bin")

        libadd.write(libadd_modified);
        binadd.write(binadd_modified)

        st = os.stat(libadd_modified)
        os.chmod(libadd_modified, st.st_mode | stat.S_IEXEC)

        st = os.stat(binadd_modified)
        os.chmod(binadd_modified, st.st_mode | stat.S_IEXEC)

        p = Popen([binadd_modified, '1', '2'], stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env={"LD_LIBRARY_PATH": self.tmp_dir})
        stdout, _ = p.communicate()

        self.assertIn('From myLIb, a + b = 3', stdout.decode("utf8"))


    @unittest.skipUnless(sys.platform.startswith("linux") and sys.version_info >= (3, 5), "requires Linux")
    def test_gnuhash(self):
        self.compile_libadd(self.libadd_path, ['-Wl,--hash-style=gnu'])
        self.compile_binadd(self.binadd_path, ['-Wl,--hash-style=gnu'])
        self.obfuscate()


    @unittest.skipUnless(sys.platform.startswith("linux") and sys.version_info >= (3, 5), "requires Linux")
    def test_sysv(self):
        self.compile_libadd(self.libadd_path, ['-Wl,--hash-style=sysv'])
        self.compile_binadd(self.binadd_path, ['-Wl,--hash-style=sysv'])
        self.obfuscate()



    def tearDown(self):
        # Delete it
        if os.path.isdir(self.tmp_dir):
            shutil.rmtree(self.tmp_dir)

if __name__ == '__main__':

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    root_logger.addHandler(ch)

    unittest.main(verbosity=2)




#!/usr/bin/env python
import unittest
import lief
import logging
import os
import io

from unittest import TestCase
from utils import get_sample

CURRENT_DIR = os.path.abspath(os.path.dirname(__file__))

def read_opcode_file(name):
    buff = None
    with open(os.path.join(CURRENT_DIR, "opcodes", name), 'r') as f:
        buff = f.read()
    buff = buff.replace("\r", "")
    return buff

class TestMachOOpcodes(TestCase):

    def setUp(self):
        self.logger = logging.getLogger(__name__)
        self.maxDiff = None

    def test_rebase_opcodes(self):
        target = lief.parse(get_sample("MachO/MachO64_x86-64_binary_rebase-LLVM.bin"))

        reference = read_opcode_file("MachO64_x86-64_binary_rebase-LLVM.rebase_opcodes")
        value = target.dyld_info.show_rebases_opcodes
        value = value.replace("\r", "")
        self.assertMultiLineEqual(reference, value)

    def test_lazy_bind_opcodes(self):
        target = lief.parse(get_sample("MachO/MachO64_x86-64_binary_lazy-bind-LLVM.bin"))

        reference = read_opcode_file("MachO64_x86-64_binary_lazy-bind-LLVM.lazy_bind_opcodes")
        value = target.dyld_info.show_lazy_bind_opcodes
        value = value.replace("\r", "")
        self.assertMultiLineEqual(reference, value)

    def test_bind_opcodes(self):
        target = lief.parse(get_sample("MachO/MachO64_x86-64_binary_lazy-bind-LLVM.bin"))

        reference = read_opcode_file("MachO64_x86-64_binary_lazy-bind-LLVM.bind_opcodes")
        value = target.dyld_info.show_bind_opcodes
        value = value.replace("\r", "")
        self.assertMultiLineEqual(reference, value)

    def test_export_trie(self):
        target = lief.parse(get_sample("MachO/MachO64_x86-64_binary_lazy-bind-LLVM.bin"))

        reference = read_opcode_file("MachO64_x86-64_binary_lazy-bind-LLVM.export_trie")
        value = target.dyld_info.show_export_trie
        value = value.replace("\r", "")
        self.assertMultiLineEqual(reference, value)




if __name__ == '__main__':

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    root_logger.addHandler(ch)

    unittest.main(verbosity=2)



#!/usr/bin/env python
import unittest
import lief
import tempfile
import sys
import subprocess
import stat
import os
import logging
import random
import itertools

from subprocess import Popen

from unittest import TestCase
from utils import get_sample

class TestMachODyld(TestCase):

    def setUp(self):
        self.logger = logging.getLogger(__name__)

    def test_exports_trie(self):
        target = lief.parse(get_sample('MachO/MachO64_x86-64_binary_exports-trie-LLVM.bin'))
        self.assertTrue(target.has_dyld_info)
        exports = target.dyld_info.exports

        self.assertEqual(len(exports), 6)

        self.assertEqual(exports[0].address, 1)
        self.assertEqual(exports[0].symbol.name, "_malloc")

        self.assertEqual(exports[1].address, 1)
        self.assertEqual(exports[1].symbol.name, "_myfree")

        self.assertEqual(exports[2].address, 0xf70)
        self.assertEqual(exports[2].symbol.name, "_myWeak")

        self.assertEqual(exports[3].address, 0x1018)
        self.assertEqual(exports[3].symbol.name, "_myTLV")

        self.assertEqual(exports[4].address, 0x12345678)
        self.assertEqual(exports[4].symbol.name, "_myAbs")

        self.assertEqual(exports[5].address, 0xf60)
        self.assertEqual(exports[5].symbol.name, "_foo")


    def test_bind(self):
        target = lief.parse(get_sample('MachO/MachO64_x86-64_binary_bind-LLVM.bin'))
        self.assertTrue(target.has_dyld_info)
        bindings = target.dyld_info.bindings

        self.assertEqual(len(bindings), 7)

        self.assertEqual(bindings[0].binding_class, lief.MachO.BINDING_CLASS.STANDARD)
        self.assertEqual(bindings[0].binding_type, lief.MachO.BIND_TYPES.POINTER)
        self.assertEqual(bindings[0].address, 0x1028)
        self.assertEqual(bindings[0].symbol.name, "_any")
        self.assertEqual(bindings[0].segment.name, "__DATA")
        self.assertEqual(bindings[0].library_ordinal, -2)

        self.assertEqual(bindings[1].binding_class, lief.MachO.BINDING_CLASS.STANDARD)
        self.assertEqual(bindings[1].binding_type, lief.MachO.BIND_TYPES.POINTER)
        self.assertEqual(bindings[1].address, 0x1020)
        self.assertEqual(bindings[1].symbol.name, "_fromApp")
        self.assertEqual(bindings[1].segment.name, "__DATA")
        self.assertEqual(bindings[1].library_ordinal, -1)

        self.assertEqual(bindings[2].binding_class, lief.MachO.BINDING_CLASS.STANDARD)
        self.assertEqual(bindings[2].binding_type, lief.MachO.BIND_TYPES.POINTER)
        self.assertEqual(bindings[2].address, 0x1018)
        self.assertEqual(bindings[2].symbol.name, "_myfunc")
        self.assertEqual(bindings[2].segment.name, "__DATA")
        self.assertEqual(bindings[2].library_ordinal, 0)

        self.assertEqual(bindings[3].binding_class, lief.MachO.BINDING_CLASS.STANDARD)
        self.assertEqual(bindings[3].binding_type, lief.MachO.BIND_TYPES.POINTER)
        self.assertEqual(bindings[3].address, 0x1000)
        self.assertEqual(bindings[3].symbol.name, "_foo")
        self.assertEqual(bindings[3].segment.name, "__DATA")
        self.assertEqual(bindings[3].library.name, "libfoo.dylib")

        self.assertEqual(bindings[4].binding_class, lief.MachO.BINDING_CLASS.STANDARD)
        self.assertEqual(bindings[4].binding_type, lief.MachO.BIND_TYPES.POINTER)
        self.assertEqual(bindings[4].address, 0x1008)
        self.assertEqual(bindings[4].symbol.name, "_bar")
        self.assertEqual(bindings[4].segment.name, "__DATA")
        self.assertEqual(bindings[4].library.name, "libbar.dylib")

        self.assertEqual(bindings[5].binding_class, lief.MachO.BINDING_CLASS.STANDARD)
        self.assertEqual(bindings[5].binding_type, lief.MachO.BIND_TYPES.POINTER)
        self.assertEqual(bindings[5].address, 0x1010)
        self.assertEqual(bindings[5].symbol.name, "_malloc")
        self.assertEqual(bindings[5].segment.name, "__DATA")
        self.assertEqual(bindings[5].library.name, "/usr/lib/libSystem.B.dylib")


        # From Weak bind
        self.assertEqual(bindings[6].binding_class, lief.MachO.BINDING_CLASS.WEAK)
        self.assertEqual(bindings[6].binding_type, lief.MachO.BIND_TYPES.POINTER)
        self.assertEqual(bindings[6].address, 0x1000)
        self.assertEqual(bindings[6].symbol.name, "_foo")
        self.assertEqual(bindings[6].segment.name, "__DATA")


    def test_lazy_bind(self):
        target = lief.parse(get_sample('MachO/MachO64_x86-64_binary_lazy-bind-LLVM.bin'))
        self.assertTrue(target.has_dyld_info)
        bindings = list(target.dyld_info.bindings)[1:] # Skip the 1st one (Standard one)
        self.assertEqual(len(bindings), 3)

        self.assertEqual(bindings[0].binding_class, lief.MachO.BINDING_CLASS.LAZY)
        self.assertEqual(bindings[0].binding_type, lief.MachO.BIND_TYPES.POINTER)
        self.assertEqual(bindings[0].address, 0x100001010)
        self.assertEqual(bindings[0].symbol.name, "_foo")
        self.assertEqual(bindings[0].segment.name, "__DATA")
        self.assertEqual(bindings[0].library.name, "libfoo.dylib")

        self.assertEqual(bindings[1].binding_class, lief.MachO.BINDING_CLASS.LAZY)
        self.assertEqual(bindings[1].binding_type, lief.MachO.BIND_TYPES.POINTER)
        self.assertEqual(bindings[1].address, 0x100001018)
        self.assertEqual(bindings[1].symbol.name, "_bar")
        self.assertEqual(bindings[1].segment.name, "__DATA")
        self.assertEqual(bindings[1].library.name, "libbar.dylib")

        self.assertEqual(bindings[2].binding_class, lief.MachO.BINDING_CLASS.LAZY)
        self.assertEqual(bindings[2].binding_type, lief.MachO.BIND_TYPES.POINTER)
        self.assertEqual(bindings[2].address, 0x100001020)
        self.assertEqual(bindings[2].symbol.name, "_malloc")
        self.assertEqual(bindings[2].segment.name, "__DATA")
        self.assertEqual(bindings[2].library.name, "/usr/lib/libSystem.B.dylib")


    def test_rebases(self):
        target = lief.parse(get_sample('MachO/MachO64_x86-64_binary_rebase-LLVM.bin'))
        self.assertTrue(target.has_dyld_info)

        relocations = target.relocations

        self.assertEqual(len(relocations), 10)

        self.assertEqual(relocations[0].address, 0x00001010)
        self.assertEqual(relocations[0].pc_relative, False)
        self.assertEqual(relocations[0].type, lief.MachO.REBASE_TYPES.POINTER)
        self.assertEqual(relocations[0].section.name, "__data")
        self.assertEqual(relocations[0].segment.name, "__DATA")

        self.assertEqual(relocations[1].address, 0x00001028)
        self.assertEqual(relocations[1].pc_relative, False)
        self.assertEqual(relocations[1].type, lief.MachO.REBASE_TYPES.POINTER)
        self.assertEqual(relocations[1].section.name, "__data")
        self.assertEqual(relocations[1].segment.name, "__DATA")

        self.assertEqual(relocations[2].address, 0x00001030)
        self.assertEqual(relocations[2].pc_relative, False)
        self.assertEqual(relocations[2].type, lief.MachO.REBASE_TYPES.POINTER)
        self.assertEqual(relocations[2].section.name, "__data")
        self.assertEqual(relocations[2].segment.name, "__DATA")

        self.assertEqual(relocations[3].address, 0x00001038)
        self.assertEqual(relocations[3].pc_relative, False)
        self.assertEqual(relocations[3].type, lief.MachO.REBASE_TYPES.POINTER)
        self.assertEqual(relocations[3].section.name, "__data")
        self.assertEqual(relocations[3].segment.name, "__DATA")

        self.assertEqual(relocations[4].address, 0x00001040)
        self.assertEqual(relocations[4].pc_relative, False)
        self.assertEqual(relocations[4].type, lief.MachO.REBASE_TYPES.POINTER)
        self.assertEqual(relocations[4].section.name, "__data")
        self.assertEqual(relocations[4].segment.name, "__DATA")

        self.assertEqual(relocations[5].address, 0x00001258)
        self.assertEqual(relocations[5].pc_relative, False)
        self.assertEqual(relocations[5].type, lief.MachO.REBASE_TYPES.POINTER)
        self.assertEqual(relocations[5].section.name, "__data")
        self.assertEqual(relocations[5].segment.name, "__DATA")


        self.assertEqual(relocations[6].address, 0x00001278)
        self.assertEqual(relocations[6].pc_relative, False)
        self.assertEqual(relocations[6].type, lief.MachO.REBASE_TYPES.POINTER)
        self.assertEqual(relocations[6].section.name, "__mystuff")
        self.assertEqual(relocations[6].segment.name, "__DATA")

        self.assertEqual(relocations[7].address, 0x00001288)
        self.assertEqual(relocations[7].pc_relative, False)
        self.assertEqual(relocations[7].type, lief.MachO.REBASE_TYPES.POINTER)
        self.assertEqual(relocations[7].section.name, "__mystuff")
        self.assertEqual(relocations[7].segment.name, "__DATA")

        self.assertEqual(relocations[8].address, 0x00001298)
        self.assertEqual(relocations[8].pc_relative, False)
        self.assertEqual(relocations[8].type, lief.MachO.REBASE_TYPES.POINTER)
        self.assertEqual(relocations[8].section.name, "__mystuff")
        self.assertEqual(relocations[8].segment.name, "__DATA")

        self.assertEqual(relocations[9].address, 0x000012A8)
        self.assertEqual(relocations[9].pc_relative, False)
        self.assertEqual(relocations[9].type, lief.MachO.REBASE_TYPES.POINTER)
        self.assertEqual(relocations[9].section.name, "__mystuff")
        self.assertEqual(relocations[9].segment.name, "__DATA")


if __name__ == '__main__':

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    root_logger.addHandler(ch)

    unittest.main(verbosity=2)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import lief
import unittest
import logging
import os
import sys
import random

from unittest import TestCase
from utils import get_sample

class TestImphash(TestCase):
    def setUp(self):
        self.logger = logging.getLogger(__name__)


    def test_without_imports(self):
        """
        By convention if a binary hasn't import, imphash is '0'
        """
        binary = lief.PE.Binary("test_imphash", lief.PE.PE_TYPE.PE32)

        self.assertEqual(int(lief.PE.get_imphash(binary), 16), 0)

    def test_casse(self):
        """
        Test that casse doesn't change the hash
        """
        binary_lhs = lief.PE.Binary("test_imphash_lhs", lief.PE.PE_TYPE.PE32)
        binary_rhs = lief.PE.Binary("test_imphash_rhs", lief.PE.PE_TYPE.PE32)

        kernel32_lhs = binary_lhs.add_library("KERNEL32.dll")
        kernel32_lhs.add_entry("CreateMutexA")

        kernel32_rhs = binary_rhs.add_library("kernel32.dll")
        kernel32_rhs.add_entry("CrEatEMutExa")

        self.assertEqual(lief.PE.get_imphash(binary_lhs), lief.PE.get_imphash(binary_rhs))


    def test_order(self):
        """
        Test that import order doesn't change the hash
        """
        binary_lhs = lief.PE.Binary("test_imphash_lhs", lief.PE.PE_TYPE.PE32)
        binary_rhs = lief.PE.Binary("test_imphash_rhs", lief.PE.PE_TYPE.PE32)
        fonctions = ["GetStringTypeW", "LCMapStringW", "GetCommandLineA", "TerminateProcess"]

        kernel32_lhs = binary_lhs.add_library("kernel32.dll")
        random.shuffle(fonctions)
        list(map(kernel32_lhs.add_entry, fonctions))
        self.logger.debug(kernel32_lhs)

        kernel32_rhs = binary_rhs.add_library("kernel32.dll")
        random.shuffle(fonctions)
        list(map(kernel32_rhs.add_entry, fonctions))
        self.logger.debug(kernel32_rhs)

        self.assertEqual(lief.PE.get_imphash(binary_lhs), lief.PE.get_imphash(binary_rhs))

    def test_ordinal(self):
        """
        Test import by ordinal
        """
        binary_lhs = lief.PE.Binary("test_imphash_lhs", lief.PE.PE_TYPE.PE32)
        binary_rhs = lief.PE.Binary("test_imphash_lhs", lief.PE.PE_TYPE.PE32)

        fonctions = [
                "GetStringTypeW",
                "LCMapStringW",
                "GetCommandLineA",
                "TerminateProcess",
                "Beep",
                "CheckRemoteDebuggerPresent",
                ]
        kernel32_lhs = binary_lhs.add_library("kernel32.dll")
        list(map(kernel32_lhs.add_entry, fonctions))

        kernel32_rhs = binary_rhs.add_library("kernel32.dll")
        for f in fonctions:
            if f == "Beep":
                imp = lief.PE.ImportEntry(0x8000001d) # Ordinal number
                kernel32_rhs.add_entry(imp)
            else:
                kernel32_rhs.add_entry(f)

        self.assertEqual(lief.PE.get_imphash(binary_lhs), lief.PE.get_imphash(binary_rhs))

    def test_order_2(self):
        """
        Test that import order doesn't change the hash (More complex)
        """
        binary_lhs = lief.PE.Binary("test_imphash_lhs", lief.PE.PE_TYPE.PE32)
        binary_rhs = lief.PE.Binary("test_imphash_rhs", lief.PE.PE_TYPE.PE32)


        libraries = {
                'KERNEL32.dll': [
                    "GetStringTypeW",
                    "LCMapStringW",
                    "GetCommandLineA",
                    "TerminateProcess",
                    "Beep",
                    "CheckRemoteDebuggerPresent",
                ],
                "ntdll.dll": [
                    "NtWriteVirtualMemory",
                    "NtYieldExecution",
                    "PfxFindPrefix",
                    "PfxInitialize",
                    "PfxInsertPrefix",
                    "PfxRemovePrefix",
                    "PropertyLengthAsVariant",
                    "RtlAbortRXact",
                ]
        }

        keys = list(libraries.keys())
        random.shuffle(keys)
        for k in keys:
            lib_lhs = binary_lhs.add_library(k)
            v = libraries[k]
            random.shuffle(v)
            for e in v:
                lib_lhs.add_entry(e)

        keys = list(libraries.keys())
        random.shuffle(keys)
        for k in keys:
            lib_rhs = binary_rhs.add_library(k)
            v = libraries[k]
            random.shuffle(v)
            for e in v:
                lib_rhs.add_entry(e)


        self.assertEqual(lief.PE.get_imphash(binary_lhs), lief.PE.get_imphash(binary_rhs))

    def test_different(self):
        """
        Check that different imports have different hashes
        """

        binary_lhs = lief.PE.Binary("test_imphash_lhs", lief.PE.PE_TYPE.PE32)
        binary_rhs = lief.PE.Binary("test_imphash_rhs", lief.PE.PE_TYPE.PE32)


        libraries = {
                'KERNEL32.dll': [
                    "GetStringTypeW",
                    "LCMapStringW",
                    "GetCommandLineA",
                    "TerminateProcess",
                    "Beep",
                    "CheckRemoteDebuggerPresent",
                ],
                "ntdll.dll": [
                    "NtWriteVirtualMemory",
                    "NtYieldExecution",
                    "PfxFindPrefix",
                    "PfxInitialize",
                    "PfxInsertPrefix",
                    "PfxRemovePrefix",
                    "PropertyLengthAsVariant",
                    "RtlAbortRXact",
                ]
        }

        keys = list(libraries.keys())
        random.shuffle(keys)
        for k in keys:
            lib_lhs = binary_lhs.add_library(k)
            v = libraries[k]
            random.shuffle(v)
            for e in v:
                lib_lhs.add_entry(e)

        keys = list(libraries.keys())
        random.shuffle(keys)
        for k in keys:
            lib_rhs = binary_rhs.add_library(k)
            v = libraries[k]
            random.shuffle(v)
            for e in filter(lambda e : len(e) % 2 == 0, v):
                lib_rhs.add_entry(e)


        self.assertNotEqual(lief.PE.get_imphash(binary_lhs), lief.PE.get_imphash(binary_rhs))





if __name__ == '__main__':

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    root_logger.addHandler(ch)

    unittest.main(verbosity=2)




#!/usr/bin/env python
import unittest
import lief
import logging
import pprint
import json
import os

from lief import Logger
Logger.set_level(lief.LOGGING_LEVEL.DEBUG)
#Logger.set_level(lief.LOGGING_LEVEL.DEBUG)

from unittest import TestCase
from utils import get_sample

CURRENT_DIR = os.path.abspath(os.path.dirname(__file__))

class TestVDEX(TestCase):

    def setUp(self):
        self.logger = logging.getLogger(__name__)


    def test_vdex06(self):
        telecom = lief.VDEX.parse(get_sample('VDEX/VDEX_06_AArch64_Telecom.vdex'))

        # 1 Dex File registred
        self.assertEqual(len(telecom.dex_files), 1)

        dex_file = telecom.dex_files[0]

        dex2dex_json_info_lhs = json.loads(dex_file.dex2dex_json_info)

        json_test_path = os.path.join(CURRENT_DIR, "VDEX_06_AArch64_Telecom_quickinfo.json")
        dex2dex_json_info_rhs = None
        #self.maxDiff = None
        with open(json_test_path, 'r') as f:
            dex2dex_json_info_rhs = json.load(f)

        self.assertEqual(dex2dex_json_info_lhs, dex2dex_json_info_rhs)

    def test_vdex10(self):
        telecom = lief.VDEX.parse(get_sample('VDEX/VDEX_10_AArch64_Telecom.vdex'))

        # 1 Dex File registred
        self.assertEqual(len(telecom.dex_files), 1)

        dex_file = telecom.dex_files[0]
        dex2dex_json_info_lhs = json.loads(dex_file.dex2dex_json_info)

        json_test_path = os.path.join(CURRENT_DIR, "VDEX_10_AArch64_Telecom_quickinfo.json")

        dex2dex_json_info_rhs = None
        self.maxDiff = None

        with open(json_test_path, 'r') as f:
            dex2dex_json_info_rhs = json.load(f)
        self.assertEqual(dex2dex_json_info_lhs, dex2dex_json_info_rhs)


class TestVDEX06(TestCase):

    def test_header(self):
        telecom = lief.VDEX.parse(get_sample('VDEX/VDEX_06_AArch64_Telecom.vdex'))
        header = telecom.header

        self.assertEqual(header.magic, [118, 100, 101, 120])
        self.assertEqual(header.version, 6)
        self.assertEqual(header.nb_dex_files, 1)
        self.assertEqual(header.dex_size, 940500)
        self.assertEqual(header.quickening_info_size, 18104)
        self.assertEqual(header.verifier_deps_size, 11580)

    def test_dex_files(self):
        telecom = lief.VDEX.parse(get_sample('VDEX/VDEX_06_AArch64_Telecom.vdex'))
        h           = hash(telecom.dex_files[0])
        h_file      = lief.hash(telecom.dex_files[0].raw(False))
        h_file_dopt = lief.hash(telecom.dex_files[0].raw(True))

        #self.assertEqual(h,           8527372568967457956)
        #self.assertEqual(h_file,      18446744072392183797)
        #self.assertEqual(h_file_dopt, 18446744073629421797)


class TestVDEX10(TestCase):

    def test_header(self):
        telecom = lief.VDEX.parse(get_sample('VDEX/VDEX_10_AArch64_Telecom.vdex'))
        header = telecom.header

        self.assertEqual(header.magic, [118, 100, 101, 120])
        self.assertEqual(header.version, 10)
        self.assertEqual(header.nb_dex_files, 1)
        self.assertEqual(header.dex_size, 1421904 )
        self.assertEqual(header.quickening_info_size, 584)
        self.assertEqual(header.verifier_deps_size, 18988)


    def test_dex_files(self):
        telecom = lief.VDEX.parse(get_sample('VDEX/VDEX_10_AArch64_Telecom.vdex'))
        h           = hash(telecom.dex_files[0])
        h_file      = lief.hash(telecom.dex_files[0].raw(False))
        h_file_dopt = lief.hash(telecom.dex_files[0].raw(True))

        #self.assertEqual(h,           4434625889427456908)
        #self.assertEqual(h_file,      18446744071715884987)
        #self.assertEqual(h_file_dopt, 18446744072171126186)



if __name__ == '__main__':

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    root_logger.addHandler(ch)

    unittest.main(verbosity=2)




#!/usr/bin/env python
import unittest
import lief
import tempfile
import sys
import subprocess
import stat
import os
import logging
import random
import itertools

from lief import Logger
Logger.set_level(lief.LOGGING_LEVEL.WARNING)
#Logger.set_level(lief.LOGGING_LEVEL.DEBUG)

from subprocess import Popen

from unittest import TestCase
from utils import get_sample

class TestELF(TestCase):

    def setUp(self):
        self.logger = logging.getLogger(__name__)

    def test_rpath(self):
        etterlog = lief.parse(get_sample('ELF/ELF64_x86-64_binary_etterlog.bin'))

        dynamic_entries = etterlog.dynamic_entries

        rpath = [e for e in dynamic_entries if e.tag == lief.ELF.DYNAMIC_TAGS.RPATH]

        self.assertEqual(len(rpath), 1)
        rpath = rpath.pop()

        self.assertEqual(rpath.name, "/usr/lib")

    def test_runpath(self):
        etterlog = lief.parse(get_sample('ELF/ELF64_x86-64_binary_systemd-resolve.bin'))

        dynamic_entries = etterlog.dynamic_entries

        runpath = [e for e in dynamic_entries if e.tag == lief.ELF.DYNAMIC_TAGS.RUNPATH]

        self.assertEqual(len(runpath), 1)
        runpath = runpath.pop()

        self.assertEqual(runpath.name, "/usr/lib/systemd")


    def test_gnuhash(self):
        ls = lief.parse(get_sample('ELF/ELF64_x86-64_binary_ls.bin'))
        gnu_hash = ls.gnu_hash

        self.assertEqual(gnu_hash.nb_buckets, 33)
        self.assertEqual(gnu_hash.symbol_index, 109)
        self.assertEqual(gnu_hash.shift2, 7)

        bloom_filters = gnu_hash.bloom_filters

        self.assertEqual(len(bloom_filters), 2)
        self.assertIn(0x3FAE01120C48A1A6, bloom_filters)
        self.assertIn(0x900004A81310D428, bloom_filters)

        buckets = gnu_hash.buckets
        self.assertEqual(len(buckets), 33)

        buckets_test = [109, 110, 0, 0, 0, 0, 0, 111, 113, 114, 0, 0, 0, 115, 0, 116, 0, 0, 117, 118, 119, 0, 120, 0, 0, 121, 123, 124, 126, 128, 129, 130, 0]
        self.assertEqual(buckets_test, buckets)


        hash_values = gnu_hash.hash_values
        hash_values_test = [0x60E0C78D, 0xF54162E5, 0x7FFD8E4E, 0x1C8BF239, 0xEEFD3EB, 0x1C8C1D29, 0x1C5871D9,
                0x5B7F3E03, 0x759A6A7F, 0xEF18DB9, 0xBA53E4D, 0x9789A097, 0x9E7650BC, 0xD39AD3D,
                0x12F7C433, 0xEB01FAB6, 0xECD54543, 0xAD3C9892, 0x72632CCF, 0x12F7A2B3, 0x7C92E3BB, 0x7C96F087]
        self.assertEqual(hash_values, hash_values_test)

        #for s in list(ls.dynamic_symbols)[gnu_hash.symbol_index:]:
        #    print(gnu_hash.check(s.name), s.name)
        self.assertTrue(all(gnu_hash.check(x.name) for x in list(ls.dynamic_symbols)[gnu_hash.symbol_index:]))

        self.assertFalse(gnu_hash.check("foofdsfdsfds"))
        self.assertFalse(gnu_hash.check("fazertrvkdfsrezklqpfjeopqdi"))

    def test_permutation(self):
        samples = [
                "ELF/ELF64_x86-64_binary_ls.bin",
                #"ELF/ELF64_x86-64_binary_gcc.bin",
                #"ELF/ELF64_x86-64_binary_openssl.bin",
        ]
        tmp_dir = tempfile.mkdtemp(suffix='_lief_test_permutation')
        for sample in samples:
            binary = lief.parse(get_sample(sample))
            dynamic_symbols = binary.dynamic_symbols

            gnu_hash_table = binary.gnu_hash

            idx = gnu_hash_table.symbol_index

            permutation = [i for i in range(1, len(dynamic_symbols))]
            random.shuffle(permutation)
            permutation = [0] + permutation
            binary.permute_dynamic_symbols(permutation)

            builder = lief.ELF.Builder(binary)
            builder.empties_gnuhash(True)
            builder.build()
            output = os.path.join(tmp_dir, "{}.permutated".format(binary.name))
            self.logger.debug("Output: {}".format(output))
            builder.write(output)

            if not sys.platform.startswith("linux"):
                return

            st = os.stat(output)
            os.chmod(output, st.st_mode | stat.S_IEXEC)

            p = Popen([output, "--help"], stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
            stdout, _ = p.communicate()
            self.logger.debug(stdout.decode("utf8"))
            self.assertEqual(p.returncode, 0)

    def test_notes(self):
        systemd_resolve = lief.parse(get_sample('ELF/ELF64_x86-64_binary_systemd-resolve.bin'))
        notes = systemd_resolve.notes
        self.assertEqual(len(notes), 3)

        n1 = notes[0]
        n2 = notes[1]
        n3 = notes[2]

        self.assertEqual(n1.name, "GNU")
        self.assertEqual(n2.name, "GNU")
        self.assertEqual(n3.name, "GNU")

        self.assertEqual(n1.type, lief.ELF.NOTE_TYPES.ABI_TAG)
        self.assertEqual(n2.type, lief.ELF.NOTE_TYPES.BUILD_ID)
        self.assertEqual(n3.type, lief.ELF.NOTE_TYPES.GOLD_VERSION)

        self.assertEqual(n1.abi, lief.ELF.NOTE_ABIS.LINUX)
        self.assertEqual(n1.version, [2, 6, 32])

        self.assertEqual(list(n2.description), [
            0x7e, 0x68, 0x6c, 0x7d,
            0x79, 0x9b, 0xa4, 0xcd,
            0x32, 0xa2, 0x34, 0xe8,
            0x4f, 0xd7, 0x45, 0x98,
            0x21, 0x32, 0x9d, 0xc8
            ])

        self.assertEqual("".join(map(chr, n3.description)), "gold 1.12")

    def test_symbols_access(self):
        hello = lief.parse(get_sample('ELF/ELF64_x86-64_binary_hello-gdb.bin'))

        symbols         = hello.symbols
        dynamic_symbols = hello.dynamic_symbols
        static_symbols  = hello.static_symbols

        self.assertTrue(all(s in symbols for s in dynamic_symbols))
        self.assertTrue(all(s in symbols for s in static_symbols))

    def test_relocation_size(self):
        aarch64_toybox = lief.parse(get_sample('ELF/ELF64_AARCH64_piebinary_toybox.pie'))
        arm_ls         = lief.parse(get_sample('ELF/ELF32_ARM_binary_ls.bin'))
        x86_ls         = lief.parse(get_sample('ELF/ELF32_x86_binary_ls.bin'))
        x86_64_ls      = lief.parse(get_sample('ELF/ELF64_x86-64_binary_ld.bin'))

        for r in itertools.chain(aarch64_toybox.dynamic_relocations, aarch64_toybox.pltgot_relocations):
            if lief.ELF.RELOCATION_AARCH64(r.type) == lief.ELF.RELOCATION_AARCH64.RELATIVE:
                self.assertEqual(r.size, 64)

            if lief.ELF.RELOCATION_AARCH64(r.type) == lief.ELF.RELOCATION_AARCH64.GLOB_DAT:
                self.assertEqual(r.size, 64)

            if lief.ELF.RELOCATION_AARCH64(r.type) == lief.ELF.RELOCATION_AARCH64.JUMP_SLOT:
                self.assertEqual(r.size, 64)

        for r in itertools.chain(arm_ls.dynamic_relocations, arm_ls.pltgot_relocations):
            if lief.ELF.RELOCATION_ARM(r.type) == lief.ELF.RELOCATION_ARM.RELATIVE:
                self.assertEqual(r.size, 32)

            if lief.ELF.RELOCATION_ARM(r.type) == lief.ELF.RELOCATION_ARM.GLOB_DAT:
                self.assertEqual(r.size, 32)

            if lief.ELF.RELOCATION_ARM(r.type) == lief.ELF.RELOCATION_ARM.ABS32:
                self.assertEqual(r.size, 32)

            if lief.ELF.RELOCATION_ARM(r.type) == lief.ELF.RELOCATION_ARM.JUMP_SLOT:
                self.assertEqual(r.size, 32)


        for r in itertools.chain(x86_ls.dynamic_relocations, x86_ls.pltgot_relocations):
            if lief.ELF.RELOCATION_i386(r.type) == lief.ELF.RELOCATION_i386.GLOB_DAT:
                self.assertEqual(r.size, 32)

            if lief.ELF.RELOCATION_i386(r.type) == lief.ELF.RELOCATION_i386.COPY:
                self.assertEqual(r.size, 32)

            if lief.ELF.RELOCATION_i386(r.type) == lief.ELF.RELOCATION_i386.JUMP_SLOT:
                self.assertEqual(r.size, 32)


        for r in itertools.chain(x86_64_ls.dynamic_relocations, x86_64_ls.pltgot_relocations):
            if lief.ELF.RELOCATION_X86_64(r.type) == lief.ELF.RELOCATION_X86_64.GLOB_DAT:
                self.assertEqual(r.size, 64)

            if lief.ELF.RELOCATION_X86_64(r.type) == lief.ELF.RELOCATION_X86_64.COPY:
                self.assertEqual(r.size, 32)

            if lief.ELF.RELOCATION_X86_64(r.type) == lief.ELF.RELOCATION_X86_64.JUMP_SLOT:
                self.assertEqual(r.size, 64)

    def test_sectionless(self):
        sample = "ELF/ELF64_x86-64_binary_rvs.bin"
        rvs = lief.parse(get_sample(sample))
        dynsym = list(rvs.dynamic_symbols)
        self.assertEqual(len(dynsym), 10)

    def test_dynamic_flags(self):
        sample = "ELF/ELF32_ARM_binary_ls.bin"
        ls = lief.parse(get_sample(sample))
        d_flags = ls.get(lief.ELF.DYNAMIC_TAGS.FLAGS)
        d_flags_1 = ls.get(lief.ELF.DYNAMIC_TAGS.FLAGS_1)

        self.assertIn(lief.ELF.DYNAMIC_FLAGS.BIND_NOW, d_flags)
        self.assertIn(lief.ELF.DYNAMIC_FLAGS_1.NOW, d_flags_1)

if __name__ == '__main__':

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    root_logger.addHandler(ch)

    unittest.main(verbosity=2)




#!/usr/bin/env python
import unittest
import lief
import tempfile
import sys
import subprocess
import stat
import os
import logging
import random
import itertools

from subprocess import Popen

from unittest import TestCase
from utils import get_sample

class TestMachO(TestCase):

    def setUp(self):
        self.logger = logging.getLogger(__name__)

    def test_function_starts(self):
        dd = lief.parse(get_sample('MachO/MachO64_x86-64_binary_dd.bin'))

        functions = [
            0x100001581, 0x1000016cc, 0x1000017cc,
            0x1000019e3, 0x100001a03, 0x100001a1d,
            0x1000020ad, 0x1000022f6, 0x1000023ef,
            0x10000246b, 0x10000248c, 0x1000026da,
            0x100002754, 0x10000286b, 0x100002914,
            0x100002bd8, 0x100002be8, 0x100002c2b,
            0x100002c62, 0x100002d24, 0x100002d5a,
            0x100002d91, 0x100002dd5, 0x100002de6,
            0x100002dfc, 0x100002e40, 0x100002e51,
            0x100002e67, 0x100002f9e
        ]

        self.assertEqual(dd.function_starts.data_offset, 21168)
        self.assertEqual(dd.function_starts.data_size,   48)
        text_segment = list(filter(lambda e : e.name == "__TEXT", dd.segments))[0]
        functions_dd = map(text_segment.virtual_address .__add__, dd.function_starts.functions)

        self.assertEqual(functions, list(functions_dd))


    def test_version_min(self):
        sshd = lief.parse(get_sample('MachO/MachO64_x86-64_binary_sshd.bin'))
        self.assertEqual(sshd.version_min.version, [10, 11, 0])
        self.assertEqual(sshd.version_min.sdk, [10, 11, 0])

    def test_va2offset(self):
        dd = lief.parse(get_sample('MachO/MachO64_x86-64_binary_dd.bin'))
        self.assertEqual(dd.virtual_address_to_offset(0x100004054), 0x4054)


    def test_thread_cmd(self):
        micromacho = lief.parse(get_sample('MachO/MachO32_x86_binary_micromacho.bin'))
        self.assertTrue(micromacho.has_thread_command)
        self.assertEqual(micromacho.thread_command.pc, 0x68)
        self.assertEqual(micromacho.thread_command.flavor, 1)
        self.assertEqual(micromacho.thread_command.count, 16)
        self.assertEqual(micromacho.entrypoint, 0x68)

    def test_rpath_cmd(self):
        rpathmacho = lief.parse(get_sample('MachO/MachO64_x86-64_binary_rpathtest.bin'))
        self.assertEqual(rpathmacho.rpath.path, "@executable_path/../lib")

    def test_relocations(self):
        helloworld = lief.parse(get_sample('MachO/MachO64_x86-64_object_HelloWorld64.o'))

        # __text Section
        text_section = helloworld.get_section("__text")
        relocations  = text_section.relocations
        self.assertEqual(len(relocations), 2)

        # 0
        self.assertEqual(relocations[0].address, 0x21b)
        self.assertEqual(relocations[0].type,    1)
        self.assertEqual(relocations[0].size,    32)

        self.assertEqual(relocations[0].is_scattered, False)

        self.assertEqual(relocations[0].has_symbol,  False)

        self.assertEqual(relocations[0].has_section,  True)
        self.assertEqual(relocations[0].section.name, text_section.name)

        # 1
        self.assertEqual(relocations[1].address, 0x233)
        self.assertEqual(relocations[1].type,    2)
        self.assertEqual(relocations[1].size,    32)

        self.assertEqual(relocations[1].is_scattered, False)

        self.assertEqual(relocations[1].has_symbol,  True)
        self.assertEqual(relocations[1].symbol.name, "_printf")

        self.assertEqual(relocations[1].has_section,  True)
        self.assertEqual(relocations[1].section.name, text_section.name)


        # __compact_unwind__LD  Section
        cunwind_section = helloworld.get_section("__compact_unwind")
        relocations  = cunwind_section.relocations
        self.assertEqual(len(relocations), 1)

        # 0
        self.assertEqual(relocations[0].address, 0x247)
        self.assertEqual(relocations[0].type,    0)
        self.assertEqual(relocations[0].size,    32)

        self.assertEqual(relocations[0].is_scattered, False)

        self.assertEqual(relocations[0].has_symbol,  False)

        self.assertEqual(relocations[0].has_section,  True)
        self.assertEqual(relocations[0].section.name, "__cstring")

    def test_data_in_code(self):
        binary = lief.parse(get_sample('MachO/MachO32_ARM_binary_data-in-code-LLVM.bin'))

        self.assertTrue(binary.has_data_in_code)
        dcode = binary.data_in_code

        self.assertEqual(dcode.data_offset, 0x11c)
        self.assertEqual(dcode.data_size, 0x20)

        self.assertEqual(len(dcode.entries), 4)

        self.assertEqual(dcode.entries[0].type, lief.MachO.DataCodeEntry.TYPES.DATA)
        self.assertEqual(dcode.entries[0].offset, 0)
        self.assertEqual(dcode.entries[0].length, 4)

        self.assertEqual(dcode.entries[1].type, lief.MachO.DataCodeEntry.TYPES.JUMP_TABLE_32)
        self.assertEqual(dcode.entries[1].offset, 4)
        self.assertEqual(dcode.entries[1].length, 4)

        self.assertEqual(dcode.entries[2].type, lief.MachO.DataCodeEntry.TYPES.JUMP_TABLE_16)
        self.assertEqual(dcode.entries[2].offset, 8)
        self.assertEqual(dcode.entries[2].length, 2)

        self.assertEqual(dcode.entries[3].type, lief.MachO.DataCodeEntry.TYPES.JUMP_TABLE_8)
        self.assertEqual(dcode.entries[3].offset, 10)
        self.assertEqual(dcode.entries[3].length, 1)


    def test_segment_split_info(self):
        binary = lief.parse(get_sample('MachO/FAT_MachO_x86_x86-64_library_libdyld.dylib'))

        self.assertTrue(binary.has_segment_split_info)
        ssi = binary.segment_split_info
        self.assertEqual(ssi.data_offset, 32852)
        self.assertEqual(ssi.data_size, 292)

    def test_dyld_environment(self):
        binary = lief.parse(get_sample('MachO/MachO64_x86-64_binary_safaridriver.bin'))
        self.assertTrue(binary.has_dyld_environment)
        self.assertEqual(binary.dyld_environment.value, "DYLD_VERSIONED_FRAMEWORK_PATH=/System/Library/StagedFrameworks/Safari")

    def test_sub_framework(self):
        binary = lief.parse(get_sample('MachO/FAT_MachO_x86_x86-64_library_libdyld.dylib'))
        self.assertTrue(binary.has_sub_framework)
        self.assertEqual(binary.sub_framework.umbrella, "System")



if __name__ == '__main__':

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    root_logger.addHandler(ch)

    unittest.main(verbosity=2)




import lief

from lief import Logger
Logger.set_level(lief.LOGGING_LEVEL.DEBUG)
from lief.ELF import Section

ls   = lief.parse("/home/romain/dev/LIEF/lief-samples/ELF/ELF64_x86-64_binary_static-binary.bin")
stub = lief.parse("hello_lief.bin")

#section            = Section()
#section.name       = "test"
#section.type       = lief.ELF.SECTION_TYPES.PROGBITS
#section.content    = stub.segments[0].content # First LOAD segment which holds payload (bytes)
#section = ls.add(section, loaded=True)
#ls.header.entrypoint = section.virtual_address + stub.header.entrypoint
#
#ep = ls.header.entrypoint

for i in range(10):
    segment                 = stub.segments[0]
    original_va             = segment.virtual_address
    segment.virtual_address = 0
    segment                 = ls.add(segment)
    new_ep                  = (stub.header.entrypoint - original_va) + segment.virtual_address

    ls.header.entrypoint = new_ep

ls.write("lst.section")

#print(hex(ep))



#!/usr/bin/env python
import signal
import unittest
import logging
import os
import sys
import stat
import re
import subprocess
import tempfile
import shutil
from subprocess import Popen

import lief
from lief.ELF import Segment
from lief import Logger
#Logger.set_level(lief.LOGGING_LEVEL.DEBUG)

from unittest import TestCase
from utils import get_sample

CURRENT_DIRECTORY = os.path.dirname(os.path.abspath(__file__))

class TestChangeInterpreter(TestCase):
    def setUp(self):
        self.logger = logging.getLogger(__name__)
        self.tmp_dir = tempfile.mkdtemp(suffix='_lief_test_add_segment')
        self.logger.debug("temp dir: {}".format(self.tmp_dir))


    @unittest.skipUnless(sys.platform.startswith("linux"), "requires Linux")
    def test_misc(self):
        list_binaries = [
        '/usr/bin/ls',
        '/usr/bin/ssh',
        '/usr/bin/nm',
        '/usr/bin/cp',
        '/usr/bin/find',
        '/usr/bin/file',
        ]
        for binary in list_binaries:
            self.logger.debug("Test with '{}'".format(binary))
            self.change_interpreter(binary)


    def change_interpreter(self, target):
        if not os.path.isfile(target):
            return

        name = os.path.basename(target)
        target = lief.parse(target)
        new_interpreter = os.path.join(self.tmp_dir, os.path.basename(target.interpreter))
        if not os.path.islink(new_interpreter):
            os.symlink(target.interpreter, new_interpreter)
        target.interpreter = new_interpreter
        output = os.path.join(self.tmp_dir, "{}.interpreter".format(name))
        target.write(output)

        st = os.stat(output)
        os.chmod(output, st.st_mode | stat.S_IEXEC)

        p = Popen(output, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        stdout, _ = p.communicate()

        self.logger.debug(stdout.decode("utf8"))
        self.assertNotEqual(p.returncode, -signal.SIGSEGV, "{} segfault!!!!".format(name))


    def tearDown(self):
        # Delete it
        if os.path.isdir(self.tmp_dir):
            shutil.rmtree(self.tmp_dir)

if __name__ == '__main__':

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    root_logger.addHandler(ch)

    unittest.main(verbosity=2)

